SorhaQuestLog = LibStub("AceAddon-3.0"):NewAddon("SorhaQuestLog", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "AceHook-3.0", "LibSink-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SorhaQuestLog")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local LSM = LibStub("LibSharedMedia-3.0")

local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local SQLBroker = nil


-- Used to stop the need to use self.db all the time
local db
local dbChar

-- Just bringing some frequently used global functions into local scope
local pairs = pairs
local tinsert = tinsert
local tContains = tContains
local tremove = tremove
local select = select
local format = format
local SecondsToTime = SecondsToTime
local GetTime = GetTime


-- Register framd font
LSM:Register("font", "framd", [[Interface\AddOns\SorhaQuestLog\Fonts\framd.ttf]])

-- Binding globals for bindings.xml
BINDING_HEADER_SORHAQUESTLOG = "Sorha Quest Log";
BINDING_NAME_TOGGLE_SORHAQUESTLOG = L["Toggle Minion"]

-- Frames
local fraQuestMinionAnchor = nil
local fraQuestMinionBorder = nil
local fraQuestTimerAnchor = nil
local fraRemoteQuestsAnchor = nil
local fraAchievementMinionAnchor = nil

-- Frame Name Prefixes
local strButtonPrefix = "SorhaQuestLogButton"
local strItemButtonPrefix = "SorhaQuestLogItemButton"
local strRemoteButtonPrefix = "SorhaQuestLogRemoteButton"

-- Frame Counts
local intNumberOfLogButtons = 0
local intNumberOfItemButtons = 0
local intNumberOfRemoteButtons = 0

-- Number Stores
local intItemButtonSize = 26
local intRemoteQuestButtonWidth = 224
local intRemoteQuestButtonHeight = 72

local tblAchievementTimers = {}
local intNumAchievementTimers = 0

-- Frame Caches
local tblItemButtonCache =  {}
local tblLogButtonCache = {}
local tblRemoteButtonCache = {}
local tblQuestMinionLogButtons = {}
local tblQuestTimerButtons = {}
local tblRemoteQuestsButtons = {}
local tblAchievementMinionButtons = {}

-- Colour strings
local strInfoColour = "|cffffffff"
local strHeaderColour = "|cffffffff"
local strQuestStatusFailed = "|cffffffff"
local strQuestStatusDone = "|cffffffff"
local strQuestStatusGoto = "|cffffffff"
local strQuestLevelColour = "|cffffffff"
local strQuestTitleColour = "|cffffffff"
local strObjectiveTitleColour = "|cffffffff"
local strObjectiveStatusColour = "|cffffffff"
local strObjective00to24Colour = "|cffffffff"
local strObjective25to49Colour = "|cffffffff"
local strObjective50to74Colour = "|cffffffff"
local strObjective75to99Colour = "|cffffffff"
local strObjective100Colour = "|cffffffff"
local strObjectiveDescriptionColour = "|cffffffff"
local strAchievementTitleColour = "|cffffffff"
local strAchievementObjectiveColour = "|cffffffff"

-- Booleans
local blnQuestMinionInitialized = false -- Quest Minion CreateLayout completed
local blnTimerInitialized = false -- Quest Timers CreateLayout completed
local blnRemoteQuestsMinionInitialized = false -- Remote Quests CreateLayout completed
local blnAchievementMinionInitialized = false -- Achievement Minion CreateLayout completed

local blnIsUpdating = false -- Minion is updating
local blnRemoteQuestsMinionIsUpdating = false
local blnAchievementMinionIsUpdating = false

local blnWasAClick = false -- Was QUEST_LOG_UPDATE called by a click on a header etc
local blnIgnoreUpdateEvents = false -- Ignores QLU events, used when making large scale collapse/expands to log
local blnFirstUpdate = true -- Was first update of quest log, no old data will be on store yet

-- Strings used to store current location for auto collapse/expand
local strZone = ""
local strSubZone = ""

-- Current and previous tables of all questlog data
local curQuestInfo = nil
local oldQuestInfo = nil

-- Time stores
local intLastOnUpdateTime = 0 -- Time OnUpdate event was last used
local intLastAchivementTimerUpdateTime = 0
local intTimeOfLastSound = 0 -- Time last sound played

-- Tables for select menus in options
local dicRepLevels = {
	["Hated"] = {["MTitle"] = FACTION_STANDING_LABEL1, ["FTitle"] = FACTION_STANDING_LABEL1_FEMALE, ["Value"] = 1.00},
	["Hostile"] = {["MTitle"] = FACTION_STANDING_LABEL2, ["FTitle"] = FACTION_STANDING_LABEL2_FEMALE, ["Value"] = 1.30},
	["Unfriendly"] = {["MTitle"] = FACTION_STANDING_LABEL3, ["FTitle"] = FACTION_STANDING_LABEL3_FEMALE, ["Value"] = 1.70},
	["Neutral"] = {["MTitle"] = FACTION_STANDING_LABEL4, ["FTitle"] = FACTION_STANDING_LABEL4_FEMALE, ["Value"] = 2.20},
	["Friendly"] = {["MTitle"] = FACTION_STANDING_LABEL5, ["FTitle"] = FACTION_STANDING_LABEL5_FEMALE, ["Value"] = 2.85},
	["Honored"] = {["MTitle"] = FACTION_STANDING_LABEL6, ["FTitle"] = FACTION_STANDING_LABEL6_FEMALE, ["Value"] = 3.71},
	["Revered"] = {["MTitle"] = FACTION_STANDING_LABEL7, ["FTitle"] = FACTION_STANDING_LABEL7_FEMALE, ["Value"] = 4.82},
	["Exalted"] = {["MTitle"] = FACTION_STANDING_LABEL8, ["FTitle"] = FACTION_STANDING_LABEL8_FEMALE, ["Value"] = 6.27},
}

local dicOutlines = {
	[""] = L["None"],
	["OUTLINE"] = L["Outline"],
	["THICKOUTLINE"] = L["Thick Outline"],
}

local dicQuestTitleColourOptions = {
	["Custom"] = L["Custom"],
	["Level"] = L["Level"],
	["Completion"] = L["Completion"],
	["Done/Undone"] = L["Done/Undone"],
}

local dicObjectiveColourOptions = {
	["Custom"] = L["Custom"],
	["Done/Undone"] = L["Done/Undone"],
	["Completion"] = L["Completion"],
}

local dicNotificationColourOptions = {
	["Custom"] = L["Custom"],
	["Completion"] = L["Completion"],
}

-- Tables used for quest tags (Group, Elite etc)
local dicQuestTags = {
	[ELITE] = "+",
	[GROUP] = "g",
	[PVP] = "p",
	[RAID] = "r",
	[LFG_TYPE_DUNGEON] = "d",
	[PLAYER_DIFFICULTY2] = "d+",
	["Daily"] = "*",
}

local dicLongQuestTags = {
	[ELITE] = ELITE,
	[GROUP] = GROUP,
	[PVP] = PVP,
	[RAID] = RAID,
	[LFG_TYPE_DUNGEON] = LFG_TYPE_DUNGEON,
	[PLAYER_DIFFICULTY2] = PLAYER_DIFFICULTY2,
	["Daily"] = L["Daily"],
}

-- Table used for matching quest completion outputs
local function getPattern(strPattern)
	strPattern = string.gsub(strPattern, "%(", "OpenBrace")
	strPattern = string.gsub(strPattern, "%)", "ClosedBrace")
	strPattern = string.gsub(strPattern, "%%.", "(.+)")
	strPattern = string.gsub(strPattern, "OpenBrace", "%%(")
	strPattern = string.gsub(strPattern, "ClosedBrace", "%%)")

	return format("^%s$", strPattern)
end

local tblQuestMatchs = {
	["Found"] = getPattern(ERR_QUEST_ADD_FOUND_SII),
	["Item"] = getPattern(ERR_QUEST_ADD_ITEM_SII),
	["Kill"] = getPattern(ERR_QUEST_ADD_KILL_SII),
	["PKill"] = getPattern(ERR_QUEST_ADD_PLAYER_KILL_SII),
	["ObjectiveComplete"] = getPattern(ERR_QUEST_OBJECTIVE_COMPLETE_S),
}


-- Addon Setup
local defaults = {
	profile = {
		Main = {
			HideAll = false,
			HideSorhaQuestLog = false,
			HideBlizzardTracker = true,
			ShowQuestTimerFrame = true,
			ShowRemoteQuestsMinionFrame = true,
			ShowAchievementMinionFrame = true,
		},
		Minion = {
			MinionWindowLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MinionWindowScale = 1,
			MinionWindowLocked = false,			
			MinionWidth = 220,
			ShowItemButtons = true,
			ShowNumberOfQuests = true,
			MoveItemsAndTooltipsRight = false,
			IndentItemButtons = false,
			IndentItemButtonQuestsOnly = false,
			CollapseToLeft = false,
			ItemButtonScale = 0.8,
			HideItemButtonsForCompletedQuests = false,
		},
		Fonts = {
			QuestFontSize = 11,
			QuestFont = "framd",
			QuestFontOutline = "",
			ObjectiveFontSize = 11,
			ObjectiveFont = "framd",
			ObjectiveFontOutline = "",
			HeaderFontSize = 11,
			HeaderFont = "framd",
			HeaderFontOutline = "",
			AchievementTitleFontSize = 11,
			AchievementTitleFont = "framd",
			AchievementTitleFontOutline = "",
			AchievementObjectiveFontSize = 11,
			AchievementObjectiveFont = "framd",
			AchievementObjectiveFontOutline = "",
		},
		ZonesAndQuests = {
			QuestLevelColouringSetting = "Level",
			QuestTitleColouringSetting = "Level",
			ObjectiveTitleColouringSetting = "Custom",
			ObjectiveStatusColouringSetting = "Completion",
			FullLengthQuestTags = false,
			HideCompletedObjectives = false,
			ShowDescWhenNoObjectives = false,
			ShowAllQuests = false,
			AllowHiddenQuests = false,
			CollapseOnLeave = false,
			ExpandOnEnter = false,
			QuestHeadersHideWhenEmpty = false,
			ShowHiddenCountOnZones = false,
			QuestTitleIndent = 5,
			ObjectivesIndent = 5,
		},
		Sounds = {
			UseQuestDoneSound = false,
			UseObjectiveDoneSound = false,
		},
		Colours = {
			InfoColour = {r = 0, g = 1, b = 0.5, a = 1},
			HeaderColour = {r = 0, g = 0.6, b = 1, a = 1},
			QuestStatusFailedColour = {r = 1, g = 1, b = 1, a = 1},
			QuestStatusDoneColour = {r = 1, g = 1, b = 1, a = 1},
			QuestStatusGotoColour = {r = 1, g = 1, b = 1, a = 1},
			QuestLevelColour = {r = 1, g = 1, b = 1, a = 1},
			QuestTitleColour = {r = 1, g = 1, b = 1, a = 1},
			ObjectiveTitleColour = {r = 1, g = 1, b = 1, a = 1},
			ObjectiveStatusColour = {r = 1, g = 1, b = 1, a = 1},
			Objective00PlusColour = {r = 1, g = 0, b = 0, a = 1},
			Objective25PlusColour = {r = 1, g = 0.3, b = 0, a = 1},
			Objective50PlusColour  = {r = 1, g = 0.6, b = 0, a = 1},
			Objective75PlusColour = {r = 1, g = 0.95, b = 0, a = 1},
			ObjectiveDoneColour = {r = 0, g = 1, b = 0, a = 1},
			ObjectiveDescColour = {r = 0.5, g = 0.5, b = 0.5, a = 0.5},
			MinionBackGroundColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			MinionBorderColour = {r = 0.5, g = 0.5, b = 0.5, a = 0},
			NotificationsColour = {r = 0, g = 1, b = 0, a = 1},
			AchievementTitleColour = {r = 0, g = 1, b = 0, a = 1},
			AchievementObjectiveColour = {r = 1, g = 1, b = 1, a = 1},
		},		
		Timer = {
			TimerWindowLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			TimerWindowScale = 1,
			TimerWindowLocked = false,
			ShowTitle = true,
			HideTitleWhenNoTimers = false,
		},
		RemoteQuests = {
			MinionLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MinionScale = 1,
			MinionLocked = false,
			ShowTitle = true,
			HideTitleWhenNoRemoteQuests = false,
		},
		AchievementMinion = {
			MinionLocation = {X = 0, Y = 0, Point = "CENTER", RelativePoint = "CENTER"},
			MinionScale = 1,
			MinionLocked = false,
			MinionWidth = 220,
			MinionCollapseToLeft = false,
			MoveTooltipsRight = false,
			ShowTitle = true,
			HideTitleWhenNoTracking = false,
		},
		Notifications = {
			UseLibSink =  false,
			LibSinkColourSetting = "Custom",
			SuppressBlizzardNotifications = false,
		},
	},
}

local function getOptions()
	if not SorhaQuestLog.options then
		SorhaQuestLog.options = {
			type = "group",
			name = "SorhaQuestLog ".. "r42",
			handler = SorhaQuestLog,
			args = {
				General = {
					name = L["General Options"],
					type = "group",
					order = 1,
					args = {
						HideAll = {
							name = L["Hide All Sorha Quest Log"],
							desc = L["Hides all of Sorha Quest Log"],
							type = "toggle",
							width = "full",
							get = function() return db.Main.HideAll end,
							set = function()
								db.Main.HideAll = not db.Main.HideAll
								SorhaQuestLog:doAnchorUpdates(false)
							end,
							order = 1,
						},
						HeaderBlizzardFrameSettingsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 50,
						},
						HeaderBlizzardFrameSettings = {
							name = L["Blizzard Frame Settings"],
							type = "header",
							order = 51,
						},
						HideBlizzardTrackerToggle = {
							name = L["Hide the default quest tracker"],
							desc = L["Hides blizzards quest tracker.. which is also used for Achievement tracking"],
							type = "toggle",
							width = "full",
							get = function() return db.Main.HideBlizzardTracker end,
							set = function()
								db.Main.HideBlizzardTracker = not db.Main.HideBlizzardTracker
								SorhaQuestLog:HandleBlizzardTracker()
							end,
							order = 52,
						},
					},
				},
				QuestMinion = {
					name = L["Quest Minion"],
					type = "group",
					order = 2,
					args = {
						MainHide = {
							name = L["Hide Minion"],
							desc = L["Hides/Shows the Main Frame"],
							type = "toggle",
							get = function() return db.Main.HideSorhaQuestLog end,
							set = function()
										db.Main.HideSorhaQuestLog = not db.Main.HideSorhaQuestLog
										SorhaQuestLog:doQuestMinionAnchorUpdate(false)
									end,
							order = 1,
						},
						MainLock = {
							name = L["Lock Frame"],
							desc = L["Unlocks/Locks the Main Frame"],
							type = "toggle",
							get = function() return db.Minion.MinionWindowLocked end,
							set = function()
										db.Minion.MinionWindowLocked = not db.Minion.MinionWindowLocked
										SorhaQuestLog:doQuestMinionAnchorUpdate(false)
									end,
							order = 2,
						},
						HeaderMainFrame = {
							name = L["Main Frame Settings"],
							type = "header",
							order = 10,
						},
						MainWidth = {
							order = 11,
							name = L["Width"],
							desc = L["Controls the width of the main window."],
							type = "range",
							min = 150, max = 600, step = 1,
							isPercent = false,
							get = function() return db.Minion.MinionWidth end,
							set = function(info, value)
								db.Minion.MinionWidth = value
								SorhaQuestLog:UpdateQuestMinion()
							end,
						},	
						Reset = {
							order = 12,
							type = "execute",
							name = L["Reset Main Frame"],
							desc = L["Resets Main Frame position"],
							func = function()
								db.Minion.MinionWindowLocation.Point = "CENTER"
								db.Minion.MinionWindowLocation.RelativePoint =  "CENTER"
								db.Minion.MinionWindowLocation.X = 0
								db.Minion.MinionWindowLocation.Y = 0
								SorhaQuestLog:doQuestMinionAnchorUpdate(true)
							end,
						},
						HeaderMiscSettingsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderMiscSettings = {
							name = L["Misc. Settings"],
							type = "header",
							order = 31,
						},
						ShowNumberQuestsToggle = {
							name = L["Show number of quests"],
							desc = L["Shows/Hides the number of quests"],
							type = "toggle",
							get = function() return db.Minion.ShowNumberOfQuests end,
							set = function()
								db.Minion.ShowNumberOfQuests = not db.Minion.ShowNumberOfQuests
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 32,
						},
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current quests is less then the max width\nNote: Doesn't work well with quests that wordwrap"],
							type = "toggle",
							get = function() return db.Minion.CollapseToLeft end,
							set = function()
								db.Minion.CollapseToLeft = not db.Minion.CollapseToLeft
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 33,
						},
						AutoTrackQuests = {
							name = L["Automatically track quests"],
							desc = L["Same as blizzard setting. Tracked quests are shown quests when the ability to hide quests is on."],
							width = "full",
							type = "toggle",
							get = function()
								if (GetCVar("autoQuestWatch") ==  "1") then
									return true
								else
									return false
								end
							end,
							set = function()
								if (GetCVar("autoQuestWatch") ==  "1") then
									SetCVar("autoQuestWatch", "0", AUTO_QUEST_WATCH_TEXT)
								else
									SetCVar("autoQuestWatch", "1", AUTO_QUEST_WATCH_TEXT)
								end
							end,
							order = 34,
						},
						AutoTrackQuestsWhenObjectiveupdate = {
							name = L["Automatically track quests when objectives update"],
							desc = L["Same as blizzard setting. Tracked quests are shown quests when the ability to hide quests is on."],
							width = "full",
							type = "toggle",
							get = function()
								if (GetCVar("autoQuestProgress") ==  "1") then
									return true
								else
									return false
								end
							end,
							set = function()
								if (GetCVar("autoQuestProgress") ==  "1") then
									SetCVar("autoQuestProgress", "0", AUTO_QUEST_PROGRESS_TEXT)
								else
									SetCVar("autoQuestProgress", "1", AUTO_QUEST_PROGRESS_TEXT)
								end
							end,
							order = 35,
						},
						HeaderItemButtonsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 70,
						},
						HeaderItemButtons = {
							name = L["Item Button Settings"],
							type = "header",
							order = 71,
						},
						ShowItemButtonsToggle = {
							name = L["Show quest item buttons"],
							desc = L["Shows/Hides the quest item buttons"],
							type = "toggle",
							get = function() return db.Minion.ShowItemButtons end,
							set = function()
								db.Minion.ShowItemButtons = not db.Minion.ShowItemButtons
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 72,
						},
						ItemsAndTooltipsRightToggle = {
							name = L["Display items and tooltips on right"],
							desc = L["Moves items and tooltips to the right"],
							type = "toggle",
							get = function() return db.Minion.MoveItemsAndTooltipsRight end,
							set = function()
								db.Minion.MoveItemsAndTooltipsRight = not db.Minion.MoveItemsAndTooltipsRight
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 73,
						},
						IndentItemButtonsToggle = {
							name = L["Indent item buttons inside tracker"],
							desc = L["Indents the item buttons into the quest tracker so they are flush with zone headers"],
							type = "toggle",
							width = "full",
							disabled = function() return (db.Minion.MoveItemsAndTooltipsRight == true or db.Minion.ShowItemButtons == false) end,
							get = function() return db.Minion.IndentItemButtons end,
							set = function()
								db.Minion.IndentItemButtons = not db.Minion.IndentItemButtons
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 74,
						},
						IndentItemButtonQuestsOnlyToggle = {
							name = L["Indent only quests with item buttons"],
							desc = L["Only indents a quest if the quest has an item button"],
							type = "toggle",
							width = "full",
							disabled = function() return (db.Minion.MoveItemsAndTooltipsRight == true or db.Minion.IndentItemButtons == false or db.Minion.ShowItemButtons == false) end,
							get = function() return db.Minion.IndentItemButtonQuestsOnly end,
							set = function()
								db.Minion.IndentItemButtonQuestsOnly = not db.Minion.IndentItemButtonQuestsOnly
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 75,
						},
						HideItemButtonsForCompletedQuestsToggle = {
							name = L["Hide Item Buttons for completed quests"],
							desc = L["Hides the quests item button once the quest is complete"],
							type = "toggle",
							width = "full",
							disabled = function() return not(db.Minion.ShowItemButtons) end,
							get = function() return db.Minion.HideItemButtonsForCompletedQuests end,
							set = function()
								db.Minion.HideItemButtonsForCompletedQuests = not db.Minion.HideItemButtonsForCompletedQuests
								SorhaQuestLog:UpdateQuestMinion()
							end,
							order = 76,
						},						
						ItemButtonsSizeSlider = {
							order = 77,
							name = L["Item Button Size"],
							desc = L["Controls the size of the Item Buttons."],
							type = "range",
							disabled = function() return not(db.Minion.ShowItemButtons) end,
							min = 0.5, max = 2, step = 0.05,
							isPercent = false,
							get = function() return db.Minion.ItemButtonScale end,
							set = function(info, value)
								db.Minion.ItemButtonScale = value
								SorhaQuestLog:UpdateQuestMinion()
							end,
						},
						QuestOptions = {
							name = L["Quest Options"],
							type = "group",
							order = 100,
							args = {
								DisplaySettingsHeader = {
									name = L["Display Settings"],
									type = "header",
									order = 1,
								},
								AllowHiddenQuestsToggle = {
									name = L["Allow quests to be hidden"],
									desc = L["Allows quests to be hidden and enables the show/hide button"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.AllowHiddenQuests end,
									set = function()
										db.ZonesAndQuests.AllowHiddenQuests = not db.ZonesAndQuests.AllowHiddenQuests
										SorhaQuestLog:doHiddenQuestsUpdate()
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 3,
								},
								AllowHiddenHeadersToggle = {
									name = L["Zone headers hide when all contained quests are hidden"],
									desc = L["Makes zone headers hide when all contained quests are hidden"],
									type = "toggle",
									disabled = function() return not db.ZonesAndQuests.AllowHiddenQuests end,
									width = "full",
									get = function() return db.ZonesAndQuests.QuestHeadersHideWhenEmpty end,
									set = function()
										db.ZonesAndQuests.QuestHeadersHideWhenEmpty = not db.ZonesAndQuests.QuestHeadersHideWhenEmpty
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 4,
								},		
								AllowHiddenCountOnZonesToggle = {
									name = L["Display count of hidden quest in each zone"],
									desc = L["Displays a count of the hidden quests in each zone on the zone header"],
									type = "toggle",
									disabled = function() return not db.ZonesAndQuests.AllowHiddenQuests end,
									width = "full",
									get = function() return db.ZonesAndQuests.ShowHiddenCountOnZones end,
									set = function()
										db.ZonesAndQuests.ShowHiddenCountOnZones = not db.ZonesAndQuests.ShowHiddenCountOnZones
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 5,
								},
								FullLengthQuestTagsToggle = {
									name = L["Display full length quest tags"],
									desc = L["Displays full length quest tags like 'Elite' instead of '+'"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.FullLengthQuestTags end,
									set = function()
										db.ZonesAndQuests.FullLengthQuestTags = not db.ZonesAndQuests.FullLengthQuestTags
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 6,
								},						
								HeaderZoneHeadersSpacer = {
									name = "   ",
									width = "full",
									type = "description",
									order = 20,
								},
								HeaderZoneHeaders = {
									name = L["Zone Header Settings"],
									type = "header",
									order = 21,
								},
								ExpandOnEnterToggle = {
									name = L["Auto expand zones on enter"],
									desc = L["Automatically expands zone headers when you enter the zone"],
									type = "toggle",
									get = function() return db.ZonesAndQuests.ExpandOnEnter end,
									set = function()
										db.ZonesAndQuests.ExpandOnEnter = not db.ZonesAndQuests.ExpandOnEnter
										SorhaQuestLog:doHandleZoneChange()
									end,
									order = 22,
								},	
								CollapseOnLeaveToggle = {
									name = L["Auto collapse zones on exit"],
									desc = L["Automatically collapses zone headers when you exit the zone"],
									type = "toggle",
									get = function() return db.ZonesAndQuests.CollapseOnLeave end,
									set = function()
										db.ZonesAndQuests.CollapseOnLeave = not db.ZonesAndQuests.CollapseOnLeave
										SorhaQuestLog:doHandleZoneChange()
									end,
									order = 23,
								},	
								HeaderFontSelect = {
									type = "select", dialogControl = 'LSM30_Font',
									order = 24,
									name = L["Header Font"],
									desc = L["The font that the zone headers will use"],
									values = AceGUIWidgetLSMlists.font,
									get = function() return db.Fonts.HeaderFont end,
									set = function(info, value)
										db.Fonts.HeaderFont = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderFontOutlineSelect = {
									name = L["Header Font Outline"],
									desc = L["The outline that the zone headers will use"],
									type = "select",
									order = 25,
									values = dicOutlines,
									get = function() return db.Fonts.HeaderFontOutline end,
									set = function(info, value)
										db.Fonts.HeaderFontOutline = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderFontSize = {
									order = 26,
									name = L["Header Font Size"],
									desc = L["Controls the font size of the header label."],
									type = "range",
									min = 10, max = 20, step = 1,
									isPercent = false,
									get = function() return db.Fonts.HeaderFontSize end,
									set = function(info, value)
										db.Fonts.HeaderFontSize = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderQuestsSpacer = {
									name = "   ",
									width = "full",
									type = "description",
									order = 40,
								},
								HeaderQuests = {
									name = L["Quest Settings"],
									type = "header",
									order = 41,
								},
								HideCompletedObjectivesToggle = {
									name = L["Hide completed objectives"],
									desc = L["Shows/Hides completed objectives"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.HideCompletedObjectives end,
									set = function()
										db.ZonesAndQuests.HideCompletedObjectives = not db.ZonesAndQuests.HideCompletedObjectives
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 42,
								},
								ShowDescWhenNoObjectivesToggle = {
									name = L["Display quest description if not objectives"],
									desc = L["Displays a quests description if there are no objectives available"],
									type = "toggle",
									width = "full",
									get = function() return db.ZonesAndQuests.ShowDescWhenNoObjectives end,
									set = function()
										db.ZonesAndQuests.ShowDescWhenNoObjectives = not db.ZonesAndQuests.ShowDescWhenNoObjectives
										SorhaQuestLog:UpdateQuestMinion()
									end,
									order = 43,
								},	
								QuestFontSelect = {
									type = "select", dialogControl = 'LSM30_Font',
									order = 44,
									name = L["Quest Font"],
									desc = L["The font that the quest titles will use"],
									values = AceGUIWidgetLSMlists.font,
									get = function() return db.Fonts.QuestFont end,
									set = function(info, value)
										db.Fonts.QuestFont = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestFontOutlineSelect = {
									name = L["Quest Font Outline"],
									desc = L["The outline that the quest titles will use"],
									type = "select",
									order = 45,
									values = dicOutlines,
									get = function() return db.Fonts.QuestFontOutline end,
									set = function(info, value)
										db.Fonts.QuestFontOutline = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestFontSize = {
									order = 46,
									name = L["Quest Font Size"],
									desc = L["Controls the font size of the quest info."],
									type = "range",
									min = 10, max = 20, step = 1,
									isPercent = false,
									get = function() return db.Fonts.QuestFontSize end,
									set = function(info, value)
										db.Fonts.QuestFontSize = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestTitleIndent = {
									order = 47,
									name = L["Quest Text Indent"],
									desc = L["Controls the level of indentation for the quest text"],
									type = "range",
									min = 0, max = 20, step = 1,
									isPercent = false,
									get = function() return db.ZonesAndQuests.QuestTitleIndent end,
									set = function(info, value)
										db.ZonesAndQuests.QuestTitleIndent = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveFontSelect = {
									type = "select", dialogControl = 'LSM30_Font',
									order = 48,
									name = L["Objective Font"],
									desc = L["The font that the quest objectives will use"],
									values = AceGUIWidgetLSMlists.font,
									get = function() return db.Fonts.ObjectiveFont end,
									set = function(info, value)
										db.Fonts.ObjectiveFont = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveFontOutlineSelect = {
									name = L["Objective Font Outline"],
									desc = L["The outline that the quest objectives will use"],
									type = "select",
									order = 49,
									values = dicOutlines,
									get = function() return db.Fonts.ObjectiveFontOutline end,
									set = function(info, value)
										db.Fonts.ObjectiveFontOutline = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveFontSize = {
									order = 50,
									name = L["Objective Font Size"],
									desc = L["Controls the font size of the quest objectives"],
									type = "range",
									min = 10, max = 20, step = 1,
									isPercent = false,
									get = function() return db.Fonts.ObjectiveFontSize end,
									set = function(info, value)
										db.Fonts.ObjectiveFontSize = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectivesIndent = {
									order = 51,
									name = L["Objective Text Indent"],
									desc = L["Controls the level of indentation for the Objective text"],
									type = "range",
									min = 0, max = 20, step = 1,
									isPercent = false,
									get = function() return db.ZonesAndQuests.ObjectivesIndent end,
									set = function(info, value)
										db.ZonesAndQuests.ObjectivesIndent = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
							},
						},
						ColourOptions = {
							name = L["Colour Options"],
							type = "group",
							order = 150,
							args = {
								HeaderColourSettings = {
									name = L["Colour Settings"],
									type = "header",
									order = 1,
								},
								QuestLevelColouringSelect = {
									name = L["Colour quest levels by:"],
									desc = L["The setting by which the colour of quest levels are determined"],
									type = "select",
									order = 2,
									values = dicQuestTitleColourOptions,
									get = function() return db.ZonesAndQuests.QuestLevelColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.QuestLevelColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								QuestTitleColouringSelect = {
									name = L["Colour quest titles by:"],
									desc = L["The setting by which the colour of quest titles is determined"],
									type = "select",
									order = 3,
									values = dicQuestTitleColourOptions,
									get = function() return db.ZonesAndQuests.QuestTitleColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.QuestTitleColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveTitleColouringSelect = {
									name = L["Colour objective title text by:"],
									desc = L["The setting by which the colour of objective title is determined"],
									type = "select",
									order = 4,
									values = dicObjectiveColourOptions,
									get = function() return db.ZonesAndQuests.ObjectiveTitleColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.ObjectiveTitleColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								ObjectiveStatusColouringSelect = {
									name = L["Colour objective status text by:"],
									desc = L["The setting by which the colour of objective statuses is determined"],
									type = "select",
									order = 5,
									values = dicObjectiveColourOptions,
									get = function() return db.ZonesAndQuests.ObjectiveStatusColouringSetting end,
									set = function(info, value)
										db.ZonesAndQuests.ObjectiveStatusColouringSetting = value
										SorhaQuestLog:UpdateQuestMinion()
									end,
								},
								HeaderMainColoursSpacer = {
									name = "   ",
									width = "full",
									type = "description",
									order = 20,
								},
								HeaderMainColours = {
									name = L["Main Colours"],
									type = "header",
									order = 21,
								},
								MinionBackGroundColour = {
									name = L["Background Colour"],
									desc = L["Sets the color of the minions background"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.MinionBackGroundColour.r = r
											db.Colours.MinionBackGroundColour.g = g
											db.Colours.MinionBackGroundColour.b = b
											db.Colours.MinionBackGroundColour.a = a
											SorhaQuestLog:UpdateQuestMinion()
										end,
									order = 22,
								},
								MinionBorderColour = {
									name = L["Border Colour"],
									desc = L["Sets the color of the minions border"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.MinionBorderColour.r = r
											db.Colours.MinionBorderColour.g = g
											db.Colours.MinionBorderColour.b = b
											db.Colours.MinionBorderColour.a = a
											SorhaQuestLog:UpdateQuestMinion()
										end,
									order = 23,
								},
								HeaderColour = {
									name = L["Zone Header Colour"],
									desc = L["Sets the color for the header of each zone"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.HeaderColour.r, db.Colours.HeaderColour.g, db.Colours.HeaderColour.b, db.Colours.HeaderColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.HeaderColour.r = r
											db.Colours.HeaderColour.g = g
											db.Colours.HeaderColour.b = b
											db.Colours.HeaderColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 24,
								},
								NoObjectivesColour = {
									name = L["No objectives description colour"],
									desc = L["Sets the color for the description displayed when there is no quest objectives"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.ObjectiveDescColour.r, db.Colours.ObjectiveDescColour.g, db.Colours.ObjectiveDescColour.b, db.Colours.ObjectiveDescColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.ObjectiveDescColour.r = r
											db.Colours.ObjectiveDescColour.g = g
											db.Colours.ObjectiveDescColour.b = b
											db.Colours.ObjectiveDescColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 25,
								},
								QuestLevelColour = {
									name = L["Quest levels"],
									desc = L["Sets the color for the quest levels if custom colouring is on"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.QuestLevelColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.QuestLevelColour.r, db.Colours.QuestLevelColour.g, db.Colours.QuestLevelColour.b, db.Colours.QuestLevelColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestLevelColour.r = r
											db.Colours.QuestLevelColour.g = g
											db.Colours.QuestLevelColour.b = b
											db.Colours.QuestLevelColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 26,
								},
								QuestTitleColour = {
									name = L["Quest titles"],
									desc = L["Sets the color for the quest titles if colouring by level is off"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.QuestTitleColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.QuestTitleColour.r, db.Colours.QuestTitleColour.g, db.Colours.QuestTitleColour.b, db.Colours.QuestTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestTitleColour.r = r
											db.Colours.QuestTitleColour.g = g
											db.Colours.QuestTitleColour.b = b
											db.Colours.QuestTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 27,
								},
								ObjectiveTitleColourPicker = {
									name = L["Objective title colour"],
									desc = L["Sets the custom color for objectives titles"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.ObjectiveTitleColour.r, db.Colours.ObjectiveTitleColour.g, db.Colours.ObjectiveTitleColour.b, db.Colours.ObjectiveTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.ObjectiveTitleColour.r = r
											db.Colours.ObjectiveTitleColour.g = g
											db.Colours.ObjectiveTitleColour.b = b
											db.Colours.ObjectiveTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 28,
								},
								ObjectiveStatusColourPicker = {
									name = L["Objective status colour"],
									desc = L["Sets the custom color for objectives statuses"],
									type = "color",
									disabled = function() return not(db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Custom") end,
									hasAlpha = true,
									get = function() return db.Colours.ObjectiveStatusColour.r, db.Colours.ObjectiveStatusColour.g, db.Colours.ObjectiveStatusColour.b, db.Colours.ObjectiveStatusColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.ObjectiveStatusColour.r = r
											db.Colours.ObjectiveStatusColour.g = g
											db.Colours.ObjectiveStatusColour.b = b
											db.Colours.ObjectiveStatusColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 29,
								},
								QuestStatusFailedColourPicker = {
									name = L["Quest failed tag"],
									desc = L["Sets the color for the quest failed tag"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.QuestStatusFailedColour.r, db.Colours.QuestStatusFailedColour.g, db.Colours.QuestStatusFailedColour.b, db.Colours.QuestStatusFailedColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestStatusFailedColour.r = r
											db.Colours.QuestStatusFailedColour.g = g
											db.Colours.QuestStatusFailedColour.b = b
											db.Colours.QuestStatusFailedColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 30,
								},								
								QuestStatusDoneColourPicker = {
									name = L["Quest done tag"],
									desc = L["Sets the color for the quest done tag"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.QuestStatusDoneColour.r, db.Colours.QuestStatusDoneColour.g, db.Colours.QuestStatusDoneColour.b, db.Colours.QuestStatusDoneColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestStatusDoneColour.r = r
											db.Colours.QuestStatusDoneColour.g = g
											db.Colours.QuestStatusDoneColour.b = b
											db.Colours.QuestStatusDoneColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 31,
								},									
								QuestStatusGotoColourPicker = {
									name = L["Quest goto Tag"],
									desc = L["Sets the color for the quest goto tag"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.QuestStatusGotoColour.r, db.Colours.QuestStatusGotoColour.g, db.Colours.QuestStatusGotoColour.b, db.Colours.QuestStatusGotoColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.QuestStatusGotoColour.r = r
											db.Colours.QuestStatusGotoColour.g = g
											db.Colours.QuestStatusGotoColour.b = b
											db.Colours.QuestStatusGotoColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 32,
								},									
							},
						},
						QuestTimer = {
							name = L["Quest Timer"],
							type = "group",
							order = 175,
							args = {
								ShowQuestTimerFrameToggle = {
									name = L["Show Quest Timer Frame"],
									desc = L["Shows a quest timer frame since the blizzard one hides with the default watcher"],
									type = "toggle",
									get = function() return db.Main.ShowQuestTimerFrame end,
									set = function()
										db.Main.ShowQuestTimerFrame = not db.Main.ShowQuestTimerFrame
										SorhaQuestLog:doQuestTimersAnchorUpdate(false)
									end,
									order = 1,
								},
								TimerWindowLockedToggle = {
									name = L["Lock Quest Timer Frame"],
									desc = L["Locks the quest timer frame"],
									type = "toggle",
									disabled = function() return not db.Main.ShowQuestTimerFrame end,
									get = function() return db.Timer.TimerWindowLocked end,
									set = function()
										db.Timer.TimerWindowLocked = not db.Timer.TimerWindowLocked
										SorhaQuestLog:doQuestTimersAnchorUpdate(false)
									end,
									order = 2,
								},
								ShowTitleToggle = {
									name = L["Show Quest Timer Frame Title"],
									desc = L["Shows the Quest Timer Frame title"],
									type = "toggle",
									width = "full",
									get = function() return db.Timer.ShowTitle end,
									set = function()
										db.Timer.ShowTitle = not db.Timer.ShowTitle
									end,
									order = 3,
								},
								HideTitleWhenNoTrackingToggle = {
									name = L["Hide title when you have no timers"],
									desc = L["Hide the Quest Timer frames title when you have no timers running"],
									type = "toggle",
									width = "full",
									disabled = function() return not(db.Timer.ShowTitle) end,
									get = function() return db.Timer.HideTitleWhenNoTimers end,
									set = function()
										db.Timer.HideTitleWhenNoTimers = not db.Timer.HideTitleWhenNoTimers
									end,
									order = 4,
								},
							},
						},
						RemoteQuests = {
							name = L["Remote Quests"],
							type = "group",
							order = 200,
							args = {
								ShowRemoteQuestMinionToggle = {
									name = L["Show Remote Quest Minion"],
									desc = L["Shows a minion for displaying remote quests since hiding the blizzard watcher hides these"],
									type = "toggle",
									get = function() return db.Main.ShowRemoteQuestsMinionFrame end,
									set = function()
										db.Main.ShowRemoteQuestsMinionFrame = not db.Main.ShowRemoteQuestsMinionFrame
										SorhaQuestLog:doRemoteQuestsMinionAnchorUpdate(false)
									end,
									order = 1,
								},								
								MinionLockedToggle = {
									name = L["Lock Remote Quests Minion"],
									desc = L["Locks the remote quests minion"],
									type = "toggle",
									disabled = function() return not db.Main.ShowQuestTimerFrame end,
									get = function() return db.RemoteQuests.MinionLocked end,
									set = function()
										db.RemoteQuests.MinionLocked = not db.RemoteQuests.MinionLocked
										SorhaQuestLog:doRemoteQuestsMinionAnchorUpdate(false)
									end,
									order = 2,
								},
								ShowTitleToggle = {
									name = L["Show Remote Quests Minion title"],
									desc = L["Shows the title for the Remote Quests Minion"],
									type = "toggle",
									width = "full",
									get = function() return db.RemoteQuests.ShowTitle end,
									set = function()
										db.RemoteQuests.ShowTitle = not db.RemoteQuests.ShowTitle
										SorhaQuestLog:UpdateRemoteQuestsMinion()
									end,
									order = 3,
								},
								HideTitleWhenNoRemoteQuestsToggle = {
									name = L["Hide title when you have no remote quests"],
									desc = L["Hide the title when you have no current remote quests"],
									type = "toggle",
									width = "full",
									disabled = function() return not(db.RemoteQuests.ShowTitle) end,
									get = function() return db.RemoteQuests.HideTitleWhenNoRemoteQuests end,
									set = function()
										db.RemoteQuests.HideTitleWhenNoRemoteQuests = not db.RemoteQuests.HideTitleWhenNoRemoteQuests
										SorhaQuestLog:UpdateRemoteQuestsMinion(false)
									end,
									order = 4,
								},
								RemoteMinionButtonSizeSlider = {
									order = 5,
									name = L["Minion Scale"],
									desc = L["Controls the size of the Minion"],
									type = "range",
									min = 0.5, max = 2, step = 0.05,
									isPercent = false,
									get = function() return db.RemoteQuests.MinionScale end,
									set = function(info, value)
										db.RemoteQuests.MinionScale = value
										SorhaQuestLog:UpdateRemoteQuestsMinion()
									end,
								},
							},
						},
					},
				},
				AchievementMinionOptions = {
					name = L["Achievement Options"],
					type = "group",
					order = 3,
					args = {
						ShowAchievementMinionFrameToggle = {
							name = L["Show Minion"],
							desc = L["Shows the Achievement Minion"],
							type = "toggle",
							get = function() return db.Main.ShowAchievementMinionFrame end,
							set = function()
								db.Main.ShowAchievementMinionFrame = not db.Main.ShowAchievementMinionFrame
								SorhaQuestLog:doAchievementMinionAnchorUpdate(false)
							end,
							order = 2,
						},					
						AchievementMinionLockedToggle = {
							name = L["Lock Minion"],
							desc = L["Locks the Achievement Minion"],
							type = "toggle",
							get = function() return db.AchievementMinion.MinionLocked end,
							set = function()
								db.AchievementMinion.MinionLocked = not db.AchievementMinion.MinionLocked
								SorhaQuestLog:doAchievementMinionAnchorUpdate(false)
							end,
							order = 3,
						},
						DisplaySettingsHeader = {
							name = L["Achievement Minion Settings"],
							type = "header",
							order = 10,
						},
						AchievementMinionWidth = {
							order = 11,
							name = L["Width"],
							desc = L["Controls the width of the minion"],
							type = "range",
							min = 150, max = 600, step = 1,
							isPercent = false,
							get = function() return db.AchievementMinion.MinionWidth end,
							set = function(info, value)
								db.AchievementMinion.MinionWidth = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},	
						AchievementMinionReset = {
							order = 12,
							type = "execute",
							name = L["Reset Minion Position"],
							desc = L["Resets Achievement Minions position"],
							func = function()
								db.AchievementMinion.MinionWindowLocation.Point = "CENTER"
								db.AchievementMinion.MinionWindowLocation.RelativePoint =  "CENTER"
								db.AchievementMinion.MinionWindowLocation.X = 0
								db.AchievementMinion.MinionWindowLocation.Y = 0
								SorhaQuestLog:doAchievementMinionAnchorUpdate(true)
							end,
						},
						HeaderMiscSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 20,
						},
						HeaderMisc = {
							name = L["Misc Settings"],
							type = "header",
							order = 21,
						},
						ShowTitleToggle = {
							name = L["Show Minion Title"],
							desc = L["Shows the Achievement Minions title"],
							type = "toggle",
							get = function() return db.AchievementMinion.ShowTitle end,
							set = function()
								db.AchievementMinion.ShowTitle = not db.AchievementMinion.ShowTitle
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 22,
						},
						HideTitleWhenNoTrackingToggle = {
							name = L["Hide title when not tracking"],
							desc = L["Hide the Achievement Minions title when not tracking"],
							type = "toggle",
							width = "full",
							disabled = function() return not(db.AchievementMinion.ShowTitle) end,
							get = function() return db.AchievementMinion.HideTitleWhenNoTracking end,
							set = function()
								db.AchievementMinion.HideTitleWhenNoTracking = not db.AchievementMinion.HideTitleWhenNoTracking
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 23,
						},
						CollapseToLeftToggle = {
							name = L["Autoshrink to left"],
							desc = L["Shrinks the width down when the length of current achivements is less then the max width\nNote: Doesn't work well with achivements that wordwrap"],
							type = "toggle",
							get = function() return db.AchievementMinion.MinionCollapseToLeft end,
							set = function()
								db.AchievementMinion.MinionCollapseToLeft = not db.AchievementMinion.MinionCollapseToLeft
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 24,
						},
						MoveTooltipsRightToggle = {
							name = L["Tooltips on right"],
							desc = L["Moves the tooltips to the right"],
							type = "toggle",
							get = function() return db.AchievementMinion.MoveTooltipsRight end,
							set = function()
								db.AchievementMinion.MoveTooltipsRight = not db.AchievementMinion.MoveTooltipsRight
								SorhaQuestLog:UpdateAchievementMinion()
							end,
							order = 25,
						},
						HeaderFontsSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 50,
						},
						HeaderFonts = {
							name = L["Font Settings"],
							type = "header",
							order = 51,
						},
						AchievementTitleFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 52,
							name = L["Title Font"],
							desc = L["The font that the achievement titles will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.AchievementTitleFont end,
							set = function(info, value)
								db.Fonts.AchievementTitleFont = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementTitleFontOutlineSelect = {
							name = L["Title Font Outline"],
							desc = L["The outline that the achievement titles will use"],
							type = "select",
							order = 53,
							values = dicOutlines,
							get = function() return db.Fonts.AchievementTitleFontOutline end,
							set = function(info, value)
								db.Fonts.AchievementTitleFontOutline = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementTitleFontSizeSize = {
							order = 54,
							name = L["Title Font Size"],
							desc = L["Controls the font size of the achievement title"],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.AchievementTitleFontSize end,
							set = function(info, value)
								db.Fonts.AchievementTitleFontSize = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						InvisibleFontsSpacer = {
							name = "",
							width = "full",
							type = "description",
							order = 55,
						},
						AchievementObjectiveFontSelect = {
							type = "select", dialogControl = 'LSM30_Font',
							order = 56,
							name = L["Task Font"],
							desc = L["The font that the achievement tasks will use"],
							values = AceGUIWidgetLSMlists.font,
							get = function() return db.Fonts.AchievementObjectiveFont end,
							set = function(info, value)
								db.Fonts.AchievementObjectiveFont = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementObjectiveFontOutlineSelect = {
							name = L["Task Font Outline"],
							desc = L["The outline that the achievement tasks will use"],
							type = "select",
							order = 57,
							values = dicOutlines,
							get = function() return db.Fonts.AchievementObjectiveFontOutline end,
							set = function(info, value)
								db.Fonts.AchievementObjectiveFontOutline = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						AchievementObjectiveFontSizeSize = {
							order = 58,
							name = L["Task Font Size"],
							desc = L["Controls the font size of the achievement task"],
							type = "range",
							min = 10, max = 20, step = 1,
							isPercent = false,
							get = function() return db.Fonts.AchievementObjectiveFontSize end,
							set = function(info, value)
								db.Fonts.AchievementObjectiveFontSize = value
								SorhaQuestLog:UpdateAchievementMinion()
							end,
						},
						ColourOptions = {
							name = L["Colour Options"],
							type = "group",
							order = 50,
							args = {
								HeaderColourSettings = {
									name = L["Colour Settings"],
									type = "header",
									order = 1,
								},
								AchievementTitleColour = {
									name = L["Achievement Titles"],
									desc = L["Sets the color for Achievement Titles"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementTitleColour.r, db.Colours.AchievementTitleColour.g, db.Colours.AchievementTitleColour.b, db.Colours.AchievementTitleColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementTitleColour.r = r
											db.Colours.AchievementTitleColour.g = g
											db.Colours.AchievementTitleColour.b = b
											db.Colours.AchievementTitleColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 2,
								},
								AchievementObjectiveColour = {
									name = L["Achievement Task"],
									desc = L["Sets the color for Achievement Objectives"],
									type = "color",
									hasAlpha = true,
									get = function() return db.Colours.AchievementObjectiveColour.r, db.Colours.AchievementObjectiveColour.g, db.Colours.AchievementObjectiveColour.b, db.Colours.AchievementObjectiveColour.a end,
									set = function(_,r,g,b,a)
											db.Colours.AchievementObjectiveColour.r = r
											db.Colours.AchievementObjectiveColour.g = g
											db.Colours.AchievementObjectiveColour.b = b
											db.Colours.AchievementObjectiveColour.a = a
											SorhaQuestLog:doColourChange()
										end,
									order = 3,
								},
							},
						},
					},
				},
				NotificationOptions = {
					name = L["Notification Settings"],
					type = "group",
					order = 5,
					args = {
						NotificationSettingsHeader = {
							name = L["Text Notification Settings"],
							type = "header",
							order = 1,
						},
						SuppressBlizzardNotificationsToggle = {
							name = L["Suppress blizzard notification messages"],
							desc = L["Suppresses the notification messages sent by blizzard to the UIErrors Frame for progress updates"],
							type = "toggle",
							width = "full",
							get = function() return db.Notifications.SuppressBlizzardNotifications end,
							set = function()
								db.Notifications.SuppressBlizzardNotifications = not db.Notifications.SuppressBlizzardNotifications
							end,
							order = 2,
						},
						UseLibSinkToggle = {
							name = L["Use LibSink for notification messages"],
							desc = L["Displays notification messages in LibSinks sink"],
							type = "toggle",
							width = "full",
							get = function() return db.Notifications.UseLibSink end,
							set = function()
								db.Notifications.UseLibSink = not db.Notifications.UseLibSink
							end,
							order = 3,
						},
						NotificationsColourSelect = {
							name = L["Lib Sink Colour by:"],
							desc = L["The setting by which the colour of notification messages are determined"],
							type = "select",
							order = 4,
							values = dicNotificationColourOptions,
							get = function() return db.Notifications.LibSinkColourSetting end,
							set = function(info, value)
								db.Notifications.LibSinkColourSetting = value
							end,
						},
						NotificationsColour = {
							name = L["Notifications"],
							desc = L["Sets the color for notifications"],
							type = "color",
							width = "full",
							hasAlpha = true,
							disabled = function() return not(db.Notifications.LibSinkColourSetting == "Custom") end,
							get = function() return db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b, db.Colours.NotificationsColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.NotificationsColour.r = r
									db.Colours.NotificationsColour.g = g
									db.Colours.NotificationsColour.b = b
									db.Colours.NotificationsColour.a = a
								end,
							order = 5,
						},
						SoundSettingsHeaderSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 20,
						},
						SoundSettingsHeader = {
							name = L["Sound Settings"],
							type = "header",
							order = 21,
						},
						UseQuestDoneSoundToggle = {
							name = L["Enable quest completed sound"],
							desc = L["Play sound when all of a quests objectives are completed"],
							type = "toggle",
							width = "full",
							get = function() return db.Sounds.UseQuestDoneSound end,
							set = function()
								db.Sounds.UseQuestDoneSound = not db.Sounds.UseQuestDoneSound
							end,
							order = 22,
						},						
						UseObjectiveDoneSoundToggle = {
							name = L["Enable objective completed sound"],
							desc = L["Play sound when a quests objective is completed"],
							type = "toggle",
							width = "full",
							get = function() return db.Sounds.UseObjectiveDoneSound end,
							set = function()
								db.Sounds.UseObjectiveDoneSound = not db.Sounds.UseObjectiveDoneSound
							end,
							order = 23,
						},
						NotificationsOptions = SorhaQuestLog:GetSinkAce3OptionsDataTable(),
					},
				},
				ColourOptions = {
					name = L["Colour Settings"],
					type = "group",
					order = 6,
					args = {
						InfoTextColour = {
							name = L["Info Text"],
							desc = L["Sets the color of the info text (Title bar, # of quests hidden etc)"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.InfoColour.r, db.Colours.InfoColour.g, db.Colours.InfoColour.b, db.Colours.InfoColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.InfoColour.r = r
									db.Colours.InfoColour.g = g
									db.Colours.InfoColour.b = b
									db.Colours.InfoColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 1,
						},
						HeaderGradualColoursSpacer = {
							name = "   ",
							width = "full",
							type = "description",
							order = 30,
						},
						HeaderGradualColours = {
							name = L["Gradual objective Colours"],
							type = "header",
							order = 31,
						},
						Objective00PlusColour = {
							name = L["0% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 0% complete"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.Objective00PlusColour.r, db.Colours.Objective00PlusColour.g, db.Colours.Objective00PlusColour.b, db.Colours.Objective00PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective00PlusColour.r = r
									db.Colours.Objective00PlusColour.g = g
									db.Colours.Objective00PlusColour.b = b
									db.Colours.Objective00PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 32,
						},
						Objective25PlusColour = {
							name = L["25% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 25% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = true,
							get = function() return db.Colours.Objective25PlusColour.r, db.Colours.Objective25PlusColour.g, db.Colours.Objective25PlusColour.b, db.Colours.Objective25PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective25PlusColour.r = r
									db.Colours.Objective25PlusColour.g = g
									db.Colours.Objective25PlusColour.b = b
									db.Colours.Objective25PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 33,
						},
						Objective50PlusColour = {
							name = L["50% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 50% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = false,
							get = function() return db.Colours.Objective50PlusColour.r, db.Colours.Objective50PlusColour.g, db.Colours.Objective50PlusColour.b, db.Colours.Objective50PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective50PlusColour.r = r
									db.Colours.Objective50PlusColour.g = g
									db.Colours.Objective50PlusColour.b = b
									db.Colours.Objective50PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 34,
						},
						Objective75PlusColour = {
							name = L["75% Complete objective colour"],
							desc = L["Sets the color for objectives that are above 75% complete"],
							type = "color",
							disabled = function() return not(db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.QuestLevelColouringSetting == "Completion" or db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") end,
							hasAlpha = true,
							get = function() return db.Colours.Objective75PlusColour.r, db.Colours.Objective75PlusColour.g, db.Colours.Objective75PlusColour.b, db.Colours.Objective75PlusColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.Objective75PlusColour.r = r
									db.Colours.Objective75PlusColour.g = g
									db.Colours.Objective75PlusColour.b = b
									db.Colours.Objective75PlusColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 35,
						},
						DoneObjectiveColour = {
							name = L["Complete objective colour"],
							desc = L["Sets the color for the complete objectives"],
							type = "color",
							hasAlpha = true,
							get = function() return db.Colours.ObjectiveDoneColour.r, db.Colours.ObjectiveDoneColour.g, db.Colours.ObjectiveDoneColour.b, db.Colours.ObjectiveDoneColour.a end,
							set = function(_,r,g,b,a)
									db.Colours.ObjectiveDoneColour.r = r
									db.Colours.ObjectiveDoneColour.g = g
									db.Colours.ObjectiveDoneColour.b = b
									db.Colours.ObjectiveDoneColour.a = a
									SorhaQuestLog:doColourChange()
								end,
							order = 36,
						},
					},
				},	
			},
		}
		SorhaQuestLog.options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(SorhaQuestLog.db)
		SorhaQuestLog.options.args.profiles.order = 9
	end
	return SorhaQuestLog.options
	
end

function SorhaQuestLog:OnInitialize() -- Called when the addon is loaded
	self.db = LibStub("AceDB-3.0"):New("SorhaQuestLogDB", defaults, true)
	db = self.db.profile
	dbChar = self.db.char
	self:SetSinkStorage(db)

	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	self:SetupOptions()
	self:SetupColours()
end

function SorhaQuestLog:OnEnable() -- Called when the addon is enabled
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()

	self:RegisterEvent("QUEST_LOG_UPDATE")	
	self:RegisterEvent('PLAYER_LEVEL_UP');
	self:RegisterEvent("ZONE_CHANGED")
	self:RegisterEvent("ZONE_CHANGED_INDOORS")
	self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
	self:RegisterEvent("TRACKED_ACHIEVEMENT_UPDATE")
	
	-- Remove achivements from blizzard questminion to skip thier error
	SetCVar("trackerFilter", "6");
		
	-- Hook for hidding blizzard quest tracker
	hooksecurefunc("WatchFrame_Update",function()
		self:HandleBlizzardTracker()
	end);
	
	-- Hook for moving quest progress messages
	self:RawHookScript(UIErrorsFrame, "OnEvent", function(self, event, msg, ...) 
		SorhaQuestLog:HandleUIErrorsFrame(self, event, msg, ...) 
	end)
	
	-- Hook for firing an update when user tracks/untracks a quest in blizzard QL
	hooksecurefunc("QuestLogTitleButton_OnClick", function(self, button)
		if (IsModifiedClick("QUESTWATCHTOGGLE")) then
			if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false and blnIsUpdating == false) then
				SorhaQuestLog:UpdateQuestMinion()
			end
		end
	end);
	
	-- Hook for remote quest entry being added
	self:RawHook("WatchFrameAutoQuest_AddPopUp", true)
	
	-- Hook for remote quest entry being removed
	self:RawHook("WatchFrameAutoQuest_ClearPopUp", true)
	
	-- LibDataBroker setup
	if LDB then
		SQLBroker = LDB:NewDataObject("SorhaQuestLog", {
			type = "data source",
			label = L["Quests"],
			icon = "Interface\\GossipFrame\\AvailableQuestIcon",	
			text = "0/25",
			OnClick = function(clickedframe, button)
				if button == "RightButton" then LibStub("AceConfigDialog-3.0"):Open("SorhaQuestLog") else SorhaQuestLog:ToggleSorhaQuestLog() end
			end,
		})
	end
	
	intTimeSinceLastSound = GetTime()
	LSM.RegisterCallback(self, "LibSharedMedia_Registered", "UpdateMedia")
	
	
	if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false) then
		self:UpdateQuestMinion()
	end
	if (db.Main.ShowQuestTimerFrame == true and db.Main.HideAll == false) then
		self:UpdateQuestTimers()	
	end
	if (db.Main.ShowRemoteQuestsMinionFrame == true and db.Main.HideAll == false) then
		self:UpdateRemoteQuestsMinion()	
	end
	if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
		self:UpdateAchievementMinion()	
	end
end

function SorhaQuestLog:SetupOptions() --Creates options. Registers chat commands
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SorhaQuestLog", getOptions())
	
	AceConfigDialog:SetDefaultSize("SorhaQuestLog", 680,525)
	self:RegisterChatCommand("sql", "ChatCommand")
	self:RegisterChatCommand("SorhaQuestLog", "ChatCommand")
end

function SorhaQuestLog:ChatCommand(input) --Handles chat commands
	if not input or input:trim() == "" then
		LibStub("AceConfigDialog-3.0"):Open("SorhaQuestLog")
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SorhaQuestLog, "SorhaQuestLog", "SorhaQuestLog", input)
	end
end


-- Event handlers
function SorhaQuestLog:QUEST_LOG_UPDATE(...)
	if (db.Main.HideSorhaQuestLog == false) then
		if (blnWasAClick == true) then -- Was the event called from a click (Skips the delay on header collapses etc)
			blnIsUpdating = true
			self:UpdateQuestMinion()
		elseif (blnIsUpdating == false) then --If not updating then update, forces a 0.5 second delay between system called updates
			if (blnIgnoreUpdateEvents == false) then
				blnIsUpdating = true
				self:ScheduleTimer("UpdateQuestMinion", 0.25)
			end
		end
	end
end

function SorhaQuestLog:PLAYER_LEVEL_UP(...)
	if (db.Main.HideSorhaQuestLog == false) then
		self:UpdateQuestMinion()
	end
end

function SorhaQuestLog:ZONE_CHANGED(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_INDOORS(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:ZONE_CHANGED_NEW_AREA(...)
	self:doHandleZoneChange()
end

function SorhaQuestLog:TRACKED_ACHIEVEMENT_UPDATE(...)
	local achievementID = select(2, ...);
	local duration = select(5, ...);
	if (duration and duration > 0) then
		tblAchievementTimers[achievementID] = {["StartTime"] = GetTime(), ["Duration"] = duration, ["TimeLeft"] = duration}
	end
	
	if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
		self:UpdateAchievementMinion()
	end
end

function SorhaQuestLog:HandleUIErrorsFrame(frame, event, msg, ...)
	if (event == "UI_INFO_MESSAGE") then
		for k, strPattern in pairs(tblQuestMatchs) do
			if (msg:match(strPattern)) then
				if (db.Notifications.UseLibSink == true) then
					if (db.Notifications.LibSinkColourSetting == "Custom") then
						self:Pour(msg, db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b)
					else
						if (strPattern == tblQuestMatchs["Found"] or strPattern == tblQuestMatchs["Item"] or strPattern == tblQuestMatchs["Kill"] or strPattern == tblQuestMatchs["PKill"]) then
							local intStart, intEnd, strText, intGot, intNeed = string.find(msg,strPattern)
							local dblPercent = intGot/intNeed
							local r,g,b
							if (dblPercent < 0.25) then
								r = db.Colours.Objective00PlusColour.r
								g = db.Colours.Objective00PlusColour.g
								b = db.Colours.Objective00PlusColour.b
							elseif (dblPercent >= 0.25 and dblPercent < 0.50) then
								r = db.Colours.Objective25PlusColour.r
								g = db.Colours.Objective25PlusColour.g
								b = db.Colours.Objective25PlusColour.b
							elseif (dblPercent >= 0.50 and dblPercent < 0.75) then
								r = db.Colours.Objective50PlusColour.r
								g = db.Colours.Objective50PlusColour.g
								b = db.Colours.Objective50PlusColour.b
							elseif (dblPercent >= 0.75 and dblPercent < 1) then
								r = db.Colours.Objective75PlusColour.r
								g = db.Colours.Objective75PlusColour.g
								b = db.Colours.Objective75PlusColour.b
							else
								r = db.Colours.ObjectiveDoneColour.r
								g = db.Colours.ObjectiveDoneColour.g
								b = db.Colours.ObjectiveDoneColour.b
							end
							self:Pour(msg, r, g, b)
						elseif (strPattern == tblQuestMatchs["ObjectiveComplete"]) then
							self:Pour(msg, db.Colours.ObjectiveDoneColour.r, db.Colours.ObjectiveDoneColour.g, db.Colours.ObjectiveDoneColour.b)
						else
							self:Pour(msg, db.Colours.NotificationsColour.r, db.Colours.NotificationsColour.g, db.Colours.NotificationsColour.b)
						end
					end
				end
				if (db.Notifications.SuppressBlizzardNotifications == true) then
					return
				end
				break
			end
		end
	end
	SorhaQuestLog.hooks[frame].OnEvent(frame, event, msg, ...)
end

function SorhaQuestLog:WatchFrameAutoQuest_AddPopUp(questId, type)
	self.hooks.WatchFrameAutoQuest_AddPopUp(questId, type)
	if (blnAchievementMinionIsUpdating == false) then
		blnAchievementMinionIsUpdating = true
		self:ScheduleTimer("UpdateRemoteQuestsMinion", 1)
	end
end

function SorhaQuestLog:WatchFrameAutoQuest_ClearPopUp(questId)
	self.hooks.WatchFrameAutoQuest_ClearPopUp(questId)
	SorhaQuestLog:UpdateRemoteQuestsMinion()
end

function SorhaQuestLog:UpdateQuestTimers()
	-- If half a second is past and timer shown, (then if timer not started create), then refresh
	if ((GetTime() - intLastOnUpdateTime) > 0.5) then
		intLastOnUpdateTime = GetTime()
		SorhaQuestLog:UpdateQuestTimerMinion(GetQuestTimers())
	end
end

function SorhaQuestLog:UpdateAchivementTimers()
	if ((GetTime() - intLastAchivementTimerUpdateTime) > 0.5) then
		intLastAchivementTimerUpdateTime = GetTime()
		if (intNumAchievementTimers > 0 and blnAchievementMinionIsUpdating == false) then
			if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
				SorhaQuestLog:UpdateAchievementMinion()
			end
		end
	end
end

function SorhaQuestLog:RefreshConfig()
	db = self.db.profile
	self:SetSinkStorage(db)
	self:doAnchorUpdates(true)
	self:doHiddenQuestsUpdate()
	self:doColourChange()
	self:HandleBlizzardTracker()
end

function SorhaQuestLog:UpdateMedia(event, mediatype, key) -- LSM getting new media
	if mediatype == "font" then
        if key == db.Fonts.QuestFont or key == db.Fonts.HeaderFont then 
			if (blnIsUpdating == false) then
				self:UpdateQuestMinion()
			end
		end
    end
end

function SorhaQuestLog:HandleBlizzardTracker()
	if (db.Main.HideBlizzardTracker == true) then
		WatchFrame:Hide()
    else
		WatchFrame:Show()
    end
end


-- Minion Creation
function SorhaQuestLog:CreateQuestMinionLayout() -- Create main minion and its buttons etc
	-- Main Anchor
	fraQuestMinionAnchor = self:doCreateFrame("FRAME","SQLQuestMinionAnchor",UIParent,db.Minion.MinionWidth,40,1,"BACKGROUND",1, db.Minion.MinionWindowLocation.Point, UIParent, db.Minion.MinionWindowLocation.RelativePoint, db.Minion.MinionWindowLocation.X, db.Minion.MinionWindowLocation.Y, 1)
	fraQuestMinionAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestMinionAnchor:SetBackdropColor(0, 1, 0, 0)
	fraQuestMinionAnchor:SetBackdropBorderColor(0.5, 0.5, 0, 0)
	
	fraQuestMinionAnchor:SetMovable(true)
	fraQuestMinionAnchor:SetClampedToScreen(true)
	fraQuestMinionAnchor:RegisterForDrag("LeftButton")
	fraQuestMinionAnchor:SetScript("OnDragStart", fraQuestMinionAnchor.StartMoving)
	fraQuestMinionAnchor:SetScript("OnDragStop",  function(self)
		fraQuestMinionAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraQuestMinionAnchor:GetPoint()
		db.Minion.MinionWindowLocation.Point = strPoint
		db.Minion.MinionWindowLocation.RelativePoint = strRelativePoint
		db.Minion.MinionWindowLocation.X = intPosX
		db.Minion.MinionWindowLocation.Y = intPosY
	end)
	
	-- Number of quests fontstring
	fraQuestMinionAnchor.objFontString = fraQuestMinionAnchor:CreateFontString(nil, "OVERLAY");
	fraQuestMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
	fraQuestMinionAnchor.objFontString:SetJustifyH("LEFT")
	fraQuestMinionAnchor.objFontString:SetJustifyV("TOP")
	fraQuestMinionAnchor.objFontString:SetText("");
	fraQuestMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraQuestMinionAnchor.objFontString:SetShadowOffset(1, -1)
	
	-- Show/Hide hidden quest button
	fraQuestMinionAnchor.buttonShowHidden = self:doCreateLooseFrame("BUTTON","SQLShowHiddenButton",fraQuestMinionAnchor,16,16,1,"LOW",1,1)
	fraQuestMinionAnchor.buttonShowHidden:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, -6);
	fraQuestMinionAnchor.buttonShowHidden:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraQuestMinionAnchor.buttonShowHidden:SetBackdropBorderColor(0, 0, 0, 0)
	
	-- Show/Hide hidden quest button hover
	fraQuestMinionAnchor.buttonShowHiddenHover = self:doCreateLooseFrame("FRAME","SQLShowHiddenButtonHover",fraQuestMinionAnchor.buttonShowHidden,fraQuestMinionAnchor.buttonShowHidden:GetWidth(),fraQuestMinionAnchor.buttonShowHidden:GetHeight(),1,"LOW",1,1)
	fraQuestMinionAnchor.buttonShowHiddenHover:SetPoint("TOPLEFT", fraQuestMinionAnchor.buttonShowHidden, "TOPLEFT", 0, 0);
	fraQuestMinionAnchor.buttonShowHiddenHover:SetBackdrop({bgFile="Interface\\AddOns\\SorhaQuestLog\\Textures\\button1_hover.tga", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile=0, tileSize=0, edgeSize=1, insets={left=-0, right=0, top=0, bottom=0}})
	fraQuestMinionAnchor.buttonShowHiddenHover:SetBackdropColor(0, 0, 1, 1)
	fraQuestMinionAnchor.buttonShowHiddenHover:SetBackdropBorderColor(0,0,0,0)
	fraQuestMinionAnchor.buttonShowHiddenHover:SetAlpha(0)
	
	-- Show/Hide hidden quest button events
	fraQuestMinionAnchor.buttonShowHidden:RegisterForClicks("AnyUp")
	fraQuestMinionAnchor.buttonShowHidden:SetScript("OnEnter", function() fraQuestMinionAnchor.buttonShowHiddenHover:SetAlpha(1) end)
	fraQuestMinionAnchor.buttonShowHidden:SetScript("OnLeave", function() fraQuestMinionAnchor.buttonShowHiddenHover:SetAlpha(0) end)
	fraQuestMinionAnchor.buttonShowHidden:SetScript("OnClick", function() 
		db.ZonesAndQuests.ShowAllQuests = not db.ZonesAndQuests.ShowAllQuests
		if (db.ZonesAndQuests.ShowAllQuests == true) then
			fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
		else
			fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
		end
		self:UpdateQuestMinion()
	end)
	
	fraQuestMinionBorder = self:doCreateFrame("FRAME","SQLQuestMinionBorder", fraQuestMinionAnchor, db.Minion.MinionWidth,40,1,"BACKGROUND",1, "TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, 0, 1)
	fraQuestMinionBorder:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestMinionBorder:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
	fraQuestMinionBorder:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	
	blnQuestMinionInitialized = true
	self:doQuestMinionAnchorUpdate(false) -- Update backcolour and visibility
	self:doHiddenQuestsUpdate()	-- Update show/hide hidden quests button position/visibility etc
end

function SorhaQuestLog:CreateQuestTimerLayout() -- Creates quest timer frame
	-- Timer anchor
	fraQuestTimerAnchor = self:doCreateFrame("FRAME","SQLQuestTimerAnchor",UIParent,100,40,1,"BACKGROUND",1, db.Timer.TimerWindowLocation.Point, UIParent, db.Timer.TimerWindowLocation.RelativePoint, db.Timer.TimerWindowLocation.X, db.Timer.TimerWindowLocation.Y, 1)
	fraQuestTimerAnchor:SetMovable(true)
	fraQuestTimerAnchor:SetClampedToScreen(true)
	fraQuestTimerAnchor:RegisterForDrag("LeftButton")
	fraQuestTimerAnchor:SetScript("OnDragStart", fraQuestTimerAnchor.StartMoving)
	fraQuestTimerAnchor:SetScript("OnUpdate", self.UpdateQuestTimers)
	fraQuestTimerAnchor:SetScript("OnDragStop",  function(self)
		fraQuestTimerAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraQuestTimerAnchor:GetPoint()
		db.Timer.TimerWindowLocation.Point = strPoint
		db.Timer.TimerWindowLocation.RelativePoint = strRelativePoint
		db.Timer.TimerWindowLocation.X = intPosX
		db.Timer.TimerWindowLocation.Y = intPosY
	end)
	fraQuestTimerAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraQuestTimerAnchor:SetBackdropColor(0, 0, 0, 0)
	fraQuestTimerAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	-- Timer fontstring for title "Quest Timer"
	fraQuestTimerAnchor.objFontString = fraQuestTimerAnchor:CreateFontString(nil, "OVERLAY");
	fraQuestTimerAnchor.objFontString:SetPoint("TOPLEFT", fraQuestTimerAnchor, "TOPLEFT",0, 0);
	fraQuestTimerAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
	fraQuestTimerAnchor.objFontString:SetJustifyH("LEFT")
	fraQuestTimerAnchor.objFontString:SetJustifyV("TOP")
	fraQuestTimerAnchor.objFontString:SetText("");
	fraQuestTimerAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraQuestTimerAnchor.objFontString:SetShadowOffset(1, -1)
	
	blnTimerInitialized = true
	self:doQuestTimersAnchorUpdate(false)
end

function SorhaQuestLog:CreateRemoteQuestsLayout() -- Creates remote quests frame
	-- Remote quests anchor
	fraRemoteQuestsAnchor = self:doCreateFrame("FRAME","SQLRemoteQuestsAnchor",UIParent,200,40,1,"BACKGROUND",1, db.RemoteQuests.MinionLocation.Point, UIParent, db.RemoteQuests.MinionLocation.RelativePoint, db.RemoteQuests.MinionLocation.X, db.RemoteQuests.MinionLocation.Y, 1)
	fraRemoteQuestsAnchor:SetMovable(true)
	fraRemoteQuestsAnchor:SetClampedToScreen(true)
	fraRemoteQuestsAnchor:RegisterForDrag("LeftButton")
	fraRemoteQuestsAnchor:SetScript("OnDragStart", fraRemoteQuestsAnchor.StartMoving)
	fraRemoteQuestsAnchor:SetScript("OnDragStop",  function(self)
		fraRemoteQuestsAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraRemoteQuestsAnchor:GetPoint()
		db.RemoteQuests.MinionLocation.Point = strPoint
		db.RemoteQuests.MinionLocation.RelativePoint = strRelativePoint
		db.RemoteQuests.MinionLocation.X = intPosX
		db.RemoteQuests.MinionLocation.Y = intPosY
	end)
	fraRemoteQuestsAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraRemoteQuestsAnchor:SetBackdropColor(0, 0, 0, 0)
	fraRemoteQuestsAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	-- Fontstring for title "Remote Quests"
	fraRemoteQuestsAnchor.objFontString = fraRemoteQuestsAnchor:CreateFontString(nil, "OVERLAY");
	fraRemoteQuestsAnchor.objFontString:SetPoint("TOPLEFT", fraRemoteQuestsAnchor, "TOPLEFT",0, 0);
	fraRemoteQuestsAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
	fraRemoteQuestsAnchor.objFontString:SetJustifyH("LEFT")
	fraRemoteQuestsAnchor.objFontString:SetJustifyV("TOP")
	fraRemoteQuestsAnchor.objFontString:SetText("");
	fraRemoteQuestsAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraRemoteQuestsAnchor.objFontString:SetShadowOffset(1, -1)
	
	blnRemoteQuestsMinionInitialized = true
	self:doRemoteQuestsMinionAnchorUpdate(false)
end

function SorhaQuestLog:CreateAchievementMinionLayout() -- Creates quest timer frame
	fraAchievementMinionAnchor = self:doCreateFrame("FRAME","SQLAchievementMinionAnchor",UIParent,100,40,1,"BACKGROUND",1, db.AchievementMinion.MinionLocation.Point, UIParent, db.AchievementMinion.MinionLocation.RelativePoint, db.AchievementMinion.MinionLocation.X, db.AchievementMinion.MinionLocation.Y, 1)
	fraAchievementMinionAnchor:SetMovable(true)
	fraAchievementMinionAnchor:SetClampedToScreen(true)
	fraAchievementMinionAnchor:RegisterForDrag("LeftButton")
	fraAchievementMinionAnchor:SetScript("OnDragStart", fraAchievementMinionAnchor.StartMoving)
	fraAchievementMinionAnchor:SetScript("OnUpdate", self.UpdateAchivementTimers)
	fraAchievementMinionAnchor:SetScript("OnDragStop",  function(self)
		fraAchievementMinionAnchor:StopMovingOrSizing()
		local strPoint, tempB, strRelativePoint, intPosX, intPosY = fraAchievementMinionAnchor:GetPoint()
		db.AchievementMinion.MinionLocation.Point = strPoint
		db.AchievementMinion.MinionLocation.RelativePoint = strRelativePoint
		db.AchievementMinion.MinionLocation.X = intPosX
		db.AchievementMinion.MinionLocation.Y = intPosY
	end)

	fraAchievementMinionAnchor:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
	fraAchievementMinionAnchor:SetBackdropColor(0, 0, 1, 0)
	fraAchievementMinionAnchor:SetBackdropBorderColor(0, 0, 0, 0)

	-- Title Fontstring
	fraAchievementMinionAnchor.objFontString = fraAchievementMinionAnchor:CreateFontString(nil, "OVERLAY");
	fraAchievementMinionAnchor.objFontString:SetPoint("TOPLEFT", fraAchievementMinionAnchor, "TOPLEFT",0, 0);
	fraAchievementMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
	fraAchievementMinionAnchor.objFontString:SetJustifyH("LEFT")
	fraAchievementMinionAnchor.objFontString:SetJustifyV("TOP")
	fraAchievementMinionAnchor.objFontString:SetText("");
	fraAchievementMinionAnchor.objFontString:SetShadowColor(0.0, 0.0, 0.0, 1.0)
	fraAchievementMinionAnchor.objFontString:SetShadowOffset(1, -1)
	
	blnAchievementMinionInitialized = true
	self:doAchievementMinionAnchorUpdate(false)
end


-- Minion Update functions
function SorhaQuestLog:UpdateQuestMinion()
	if (blnQuestMinionInitialized == false) then
		self:CreateQuestMinionLayout()
	end
	
	-- Setup variables
	local intSpacingIncrease = 0
	local intLargestWidth = 0
	local blnLargestWidthIsHeader = false
	local intHeaderOutlineOffset = 0
	local intQuestOutlineOffset = 0
	local intObjectiveOutlineOffset = 0
	local intYPosition = 0
	local intButtonSize = intItemButtonSize * db.Minion.ItemButtonScale
	local intQuestOffset =  db.ZonesAndQuests.QuestTitleIndent
	local intObjectiveOffset =  db.ZonesAndQuests.ObjectivesIndent
	local intQuestWithItemButtonOffset = intQuestOffset + intButtonSize / 2
	
	-- Reset was click to update boolean		
	blnWasAClick = false
		
	-- Release all buttons in use
	for k, objButton in pairs(tblQuestMinionLogButtons) do
		self:RecycleLogButton(objButton)
	end
	wipe(tblQuestMinionLogButtons)

	-- If this isn't the first get of quest info store the current info for later use
	if not(curQuestInfo == nil) then
		oldQuestInfo = curQuestInfo
		blnFirstUpdate = false
	end
	curQuestInfo = self:GetQuestLogInformation()

	-- Quest/Objective complete Sounds
	if (blnFirstUpdate == false and (db.Sounds.UseQuestDoneSound == true or db.Sounds.UseObjectiveDoneSound == true)) then
		if ((GetTime() - intTimeOfLastSound) > 1) then
			local strSound = self:CheckQuestStateChange()
			if (strSound == "QuestDone" and db.Sounds.UseQuestDoneSound == true) then
				PlaySoundFile("Sound\\Creature\\Peon\\PeonBuildingComplete1.wav");
				intTimeOfLastSound = GetTime()
			elseif (strSound == "ObjectiveDone" and db.Sounds.UseObjectiveDoneSound == true) then
				PlaySoundFile("Sound\\Creature\\Peasant\\PeasantWhat3.wav");
				intTimeOfLastSound = GetTime()
			else
				--Nothing
			end
		end
	end	
	
	--Add in slight offsets for outlined text to try stop overlap
	if (db.Fonts.HeaderFontOutline == "THICKOUTLINE") then
		intHeaderOutlineOffset = 2
	elseif (db.Fonts.HeaderFontOutline == "OUTLINE") then
		intHeaderOutlineOffset = 1
	end
	if (db.Fonts.QuestFontOutline == "THICKOUTLINE") then
		intQuestOutlineOffset = 1.5
	elseif (db.Fonts.QuestFontOutline == "OUTLINE") then
		intQuestOutlineOffset = 0.5
	end
	if (db.Fonts.ObjectiveFontOutline == "THICKOUTLINE") then
		intObjectiveOutlineOffset = 1.5
	elseif (db.Fonts.ObjectiveFontOutline == "OUTLINE") then
		intObjectiveOutlineOffset = 0.5
	end	
		
	-- Number of quests title display
	if (db.Minion.ShowNumberOfQuests == true) then
		fraQuestMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 14, db.Fonts.HeaderFontOutline)
		fraQuestMinionAnchor.objFontString:SetText(strInfoColour .. curQuestInfo["NoQuests"] .. "/25|r");
		if (fraQuestMinionAnchor.objFontString:GetWidth() > intLargestWidth) then
			if (db.ZonesAndQuests.AllowHiddenQuests == true) then
				intLargestWidth = fraQuestMinionAnchor.objFontString:GetWidth() + 20 -- Offset for the show/hide button pushing the fontstring accross
			else
				intLargestWidth = fraQuestMinionAnchor.objFontString:GetWidth()
			end
			
		end
	else
		fraQuestMinionAnchor.objFontString:SetText("");
	end
	fraQuestMinionAnchor:SetWidth(db.Minion.MinionWidth)
	
	-- Update LDB 
	if LDB then
		SQLBroker.text = strInfoColour .. curQuestInfo["NoQuests"] .. "/25|r"
	end
	
	-- Zone/Quest buttons
	local objButton = nil
	for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
		if not((ZoneInstance["NumHidden"] == ZoneInstance["NumQuests"]) and (ZoneInstance["HasHidden"] == true) and db.ZonesAndQuests.QuestHeadersHideWhenEmpty == true and db.ZonesAndQuests.ShowAllQuests == false and db.ZonesAndQuests.AllowHiddenQuests == true) then
			objButton = self:GetQuestMinionButton()
			objButton.LogPosition = ZoneInstance["LogPosition"]
			objButton.isHeader = 1
			objButton.isCollapsed = ZoneInstance["Status"]
			objButton.intOffset = 0
	
			local strPrefix = strHeaderColour .. "- "
			if (ZoneInstance["Status"] == 1) then
				strPrefix = strHeaderColour .. "+ "
			end

			objButton:SetPoint("TOP", fraQuestMinionAnchor, "CENTER", 0, intYPosition - intHeaderOutlineOffset)
			objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
			objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), db.Fonts.HeaderFontSize, db.Fonts.HeaderFontOutline)
			if (db.ZonesAndQuests.ShowHiddenCountOnZones == true and db.ZonesAndQuests.AllowHiddenQuests == true) then
				if (ZoneInstance["NumHidden"] > 0) then
					objButton.objFontString1:SetText(strPrefix .. ZoneInstance["Title"] .. "|r (" .. strInfoColour .. ZoneInstance["NumHidden"] .. "/" .. ZoneInstance["NumQuests"] .." Hidden)|r");	
				else
					objButton.objFontString1:SetText(strPrefix .. ZoneInstance["Title"] .. "|r");
				end
			else
				objButton.objFontString1:SetText(strPrefix .. ZoneInstance["Title"] .. "|r");
			end
			
			objButton:SetWidth(db.Minion.MinionWidth)
			--
			if (objButton.objFontString1:GetWidth() > intLargestWidth) then
				intLargestWidth = objButton.objFontString1:GetWidth()
				blnLargestWidthIsHeader = true
			end
			--
			objButton.objFontString1:SetWidth(db.Minion.MinionWidth)
			
			intSpacingIncrease = objButton.objFontString1:GetHeight() + intHeaderOutlineOffset
			objButton:SetHeight(intSpacingIncrease)
			tinsert(tblQuestMinionLogButtons, objButton)

			intYPosition = intYPosition - intSpacingIncrease

			-- Create each quest in zone
			for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
				if (QuestInstance["IsHidden"] == false or db.ZonesAndQuests.AllowHiddenQuests == false or db.ZonesAndQuests.ShowAllQuests == true) then

					local blnHasShownButton = false
					local intThisQuestsOffset = intQuestOffset
					if (db.Minion.ShowItemButtons == true and QuestInstance["QuestItem"]) then -- Item Buttons on and has a button
						if (db.Minion.HideItemButtonsForCompletedQuests == false or (db.Minion.HideItemButtonsForCompletedQuests == true and not(QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0))) then -- Button not hidden because of completion
							blnHasShownButton = true
							if (db.Minion.IndentItemButtons == true and db.Minion.MoveItemsAndTooltipsRight == false) then
								intThisQuestsOffset = intQuestWithItemButtonOffset
							end
						end
					end
					if (db.Minion.IndentItemButtonQuestsOnly == false and db.Minion.IndentItemButtons == true and db.Minion.MoveItemsAndTooltipsRight == false) then
						intThisQuestsOffset = intQuestWithItemButtonOffset				
					end

					objButton = self:GetQuestMinionButton()
					objButton.LogPosition = QuestInstance["LogPosition"]
					objButton:SetPoint("TOP", fraQuestMinionAnchor, "CENTER", intThisQuestsOffset, intYPosition)
					objButton.intOffset = intThisQuestsOffset
					
					-- Get quest text
					tblQuestText = self:GetQuestText(QuestInstance)
					
					-- Setup quest title string
					objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
					objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.QuestFont), db.Fonts.QuestFontSize, db.Fonts.QuestFontOutline)
					objButton.objFontString1:SetText(tblQuestText["Title"]);

					-- Setup quest objectives string
					objButton.objFontString2:SetFont(LSM:Fetch("font", db.Fonts.ObjectiveFont), db.Fonts.ObjectiveFontSize, db.Fonts.ObjectiveFontOutline)
					objButton.objFontString2:SetText(tblQuestText["Objectives"]);			
					
					objButton:SetWidth(db.Minion.MinionWidth - intThisQuestsOffset*2)

					-- Find out if either string is larger then the current largest string
					if (objButton.objFontString1:GetWidth() + intThisQuestsOffset*2 > intLargestWidth) then
						intLargestWidth = objButton.objFontString1:GetWidth() + intThisQuestsOffset*2
						blnLargestWidthIsHeader = false
					end
					if (objButton.objFontString2:GetWidth() + (intThisQuestsOffset*2 + intObjectiveOffset) > intLargestWidth) then
						intLargestWidth = objButton.objFontString2:GetWidth() + intThisQuestsOffset*2 + intObjectiveOffset
						blnLargestWidthIsHeader = false
					end

					objButton.objFontString1:SetWidth(db.Minion.MinionWidth - intThisQuestsOffset*2)
					
					-- Set second fontstring of the buttons position
					local intSecondFSOffset = objButton.objFontString1:GetHeight() + intQuestOutlineOffset
					objButton.objFontString2:SetPoint("TOPLEFT", objButton, "TOPLEFT", intObjectiveOffset, -intSecondFSOffset);
					objButton.objFontString2:SetWidth(db.Minion.MinionWidth - intThisQuestsOffset*2 - intObjectiveOffset)
					
					-- Find spacing needed for next button
					intSpacingIncrease = objButton.objFontString2:GetHeight() + intObjectiveOutlineOffset + intSecondFSOffset				
				
					-- If theres an item button to be shown add it
					if (blnHasShownButton == true) then
						local objItem = QuestInstance["QuestItem"]
						local objItemButton = self:GetItemButton(objItem, QuestInstance["LogPosition"])
						objItemButton:SetParent(objButton)

						if (db.Minion.MoveItemsAndTooltipsRight == true) then
							objItemButton:SetPoint("TOPLEFT", objButton, "TOPRIGHT", 4, 0)
						else
							if (db.Minion.IndentItemButtons == true) then
								objItemButton:SetPoint("TOPRIGHT", objButton, "TOPLEFT", 0, 0)
							else
								objItemButton:SetPoint("TOPRIGHT", objButton, "TOPLEFT", -(16 * (1 / objItemButton:GetScale())), 0)
							end
						end
						
						-- If a button is heigher then its quest then expand the quest frame to stop overlapping buttons
						if (intButtonSize > intSpacingIncrease) then
							intSpacingIncrease = intButtonSize
							objButton.objFontString1:SetHeight(intButtonSize)
						end
						objButton.ItemButton = objItemButton
					end
					objButton:SetHeight(intSpacingIncrease)
					tinsert(tblQuestMinionLogButtons, objButton)

					intYPosition = intYPosition - intSpacingIncrease
				end
			end
		end
	end

	-- Auto collapse
	if (db.Minion.CollapseToLeft == true) then
		if (intLargestWidth < db.Minion.MinionWidth) then
			fraQuestMinionAnchor:SetWidth(intLargestWidth)
			
			for k, objButton in pairs(tblQuestMinionLogButtons) do
				objButton.objFontString1:SetWidth(intLargestWidth - objButton.intOffset * 2)
				objButton:SetWidth(intLargestWidth - objButton.intOffset * 2)
			end
		end
	end
	
	-- Border/Background
	fraQuestMinionBorder:SetBackdropColor(db.Colours.MinionBackGroundColour.r, db.Colours.MinionBackGroundColour.g, db.Colours.MinionBackGroundColour.b, db.Colours.MinionBackGroundColour.a)
	fraQuestMinionBorder:SetBackdropBorderColor(db.Colours.MinionBorderColour.r, db.Colours.MinionBorderColour.g, db.Colours.MinionBorderColour.b, db.Colours.MinionBorderColour.a)
	fraQuestMinionBorder:ClearAllPoints()
	
	fraQuestMinionBorder:SetWidth(fraQuestMinionAnchor:GetWidth() + 12)
	fraQuestMinionBorder:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", -6, 0)
	fraQuestMinionBorder:SetHeight((-intYPosition) + 10 + fraQuestMinionAnchor:GetHeight()/2)	
	
	blnIsUpdating = false
end

function SorhaQuestLog:UpdateQuestTimerMinion(...) -- Refresh timer minion
	local numTimers = select("#", ...);
	local dblYPosition = db.Fonts.HeaderFontSize
	local dblOffset = 0
	
	-- If QuestTimer minion not initalized then create it
	if (blnTimerInitialized == false) then
		self:CreateQuestTimerLayout()
	end	
	
	-- Release all used buttons
	for k, objButton in pairs(tblQuestTimerButtons) do
		self:RecycleLogButton(objButton)
	end
	wipe(tblQuestTimerButtons)
	
	-- Create timers
	for i = 1, numTimers do
		objButton = self:GetLogButton()
		objButton:SetPoint("TOPLEFT", fraQuestTimerAnchor, "TOPLEFT", 0, -dblYPosition);
		
		-- Set buttons text
		objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.QuestFont), db.Fonts.QuestFontSize, db.Fonts.QuestFontOutline)
		objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
		objButton.objFontString1:SetText(" - " .. SecondsToTime(select(i, ...)))
		
		-- Get offset for next button
		dblOffset = objButton.objFontString1:GetStringHeight()	
		
		-- Set Height and Width
		objButton:SetHeight(dblOffset)
		objButton:SetWidth(objButton.objFontString1:GetStringWidth())
		
		-- Setup scripts
		objButton:SetScript("OnEnter", function (self) GameTooltip:SetOwner(self); GameTooltip:SetHyperlink(GetQuestLink(GetQuestIndexForTimer(i))); GameTooltip:Show(); end);
		objButton:SetScript("OnLeave", GameTooltip_Hide);
		
		dblYPosition = dblYPosition + dblOffset
		tinsert(tblQuestTimerButtons, objButton)
	end
	
	-- Show title if enabled
	if (db.Timer.ShowTitle == true and (db.Timer.HideTitleWhenNoTimers == false or (db.Timer.HideTitleWhenNoTimers == true and (numTimers > 0)))) then
		fraQuestTimerAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), db.Fonts.HeaderFontSize, db.Fonts.HeaderFontOutline)
		fraQuestTimerAnchor.objFontString:SetText(strInfoColour .. L["Quest Timer Frame Title"])
	else
		fraQuestTimerAnchor.objFontString:SetText("")
	end			
end

function SorhaQuestLog:UpdateRemoteQuestsMinion()
	blnAchievementMinionIsUpdating = true
	
	-- If Achievement Minion is not Initialized then do so
	if (blnRemoteQuestsMinionInitialized == false) then
		self:CreateRemoteQuestsLayout()
	end
	
	-- Release all used buttons
	for k, objButton in pairs(tblRemoteQuestsButtons) do
		self:RecycleRemoteQuestButton(objButton)
	end
	wipe(tblRemoteQuestsButtons)
	
	-- Get Number of Auto Quests
	local numAutoQuestPopUps = GetNumAutoQuestPopUps();
	
	-- Show title if enabled
	if (db.RemoteQuests.ShowTitle == true and (db.RemoteQuests.HideTitleWhenNoRemoteQuests == false or (db.RemoteQuests.HideTitleWhenNoRemoteQuests == true and (numAutoQuestPopUps > 0)))) then
		fraRemoteQuestsAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), db.Fonts.HeaderFontSize, db.Fonts.HeaderFontOutline)
		fraRemoteQuestsAnchor.objFontString:SetText(strInfoColour .. L["Remote Quests Minion Title"])
	else
		fraRemoteQuestsAnchor.objFontString:SetText("")
	end			
	
	local intYPosition = (db.Fonts.HeaderFontSize / db.RemoteQuests.MinionScale)
	for i=1, numAutoQuestPopUps do
		local questID, popUpType = GetAutoQuestPopUp(i);
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, _ = GetQuestLogTitle(GetQuestLogIndexByID(questID));
		
		if ( isComplete and isComplete > 0 ) then
			isComplete = true;
		else
			isComplete = false;
		end	

		if (questTitle and questTitle ~= "") then
			local objButton = self:GetRemoteQuestButton();
			objButton:SetScale(db.RemoteQuests.MinionScale)
			objButton:SetParent(fraRemoteQuestsAnchor);
			objButton.index = i;

			if (isComplete and popUpType == "COMPLETE") then
				objButton.ScrollChild.QuestionMark:Show();
				objButton.ScrollChild.Exclamation:Hide();
				objButton.ScrollChild.TopText:SetText(QUEST_WATCH_POPUP_CLICK_TO_COMPLETE);
				objButton.ScrollChild.BottomText:Hide();
				objButton.ScrollChild.TopText:SetPoint("TOP", 0, -12);
				objButton.ScrollChild.QuestName:SetPoint("TOP", 0, -32);
				if (objButton.questId and objButton.type=="OFFER") then
					objButton.ScrollChild.Flash:Show();
				end
				objButton.type="COMPLETED";
			elseif (popUpType == "OFFER") then
				objButton.ScrollChild.QuestionMark:Hide();
				objButton.ScrollChild.Exclamation:Show();
				objButton.ScrollChild.TopText:SetText(QUEST_WATCH_POPUP_QUEST_DISCOVERED);
				objButton.ScrollChild.BottomText:Show();
				objButton.ScrollChild.BottomText:SetText(QUEST_WATCH_POPUP_CLICK_TO_VIEW);
				objButton.ScrollChild.TopText:SetPoint("TOP", 0, -4);
				objButton.ScrollChild.QuestName:SetPoint("TOP", 0, -24);
				objButton.ScrollChild.Flash:Hide();
				objButton.type="OFFER";
			end
			objButton:SetPoint("TOPLEFT", fraRemoteQuestsAnchor, "TOPLEFT", 0, -intYPosition);

			objButton.ScrollChild.QuestName:SetText(questTitle);
			objButton.questId = questID;
			
			objButton.ScrollChild.Flash:Hide();
			objButton.totalTime = 0;
			objButton.slideInTime = 0.5;
			objButton:SetHeight(1);
			objButton:SetScript("OnUpdate", RemoteQuestButton_OnUpdate);
			
			intYPosition = intYPosition + intRemoteQuestButtonHeight
			
			tinsert(tblRemoteQuestsButtons,objButton)			
		end
	end
	
	
	blnAchievementMinionIsUpdating = false
end

function SorhaQuestLog:UpdateAchievementMinion()
	blnAchievementMinionIsUpdating = true

	-- If Achievement Minion is not Initialized then do so
	if (blnAchievementMinionInitialized == false) then
		self:CreateAchievementMinionLayout()
	end
	
	-- Release all used buttons
	for k, objButton in pairs(tblAchievementMinionButtons) do
		self:RecycleLogButton(objButton)
	end
	wipe(tblAchievementMinionButtons)
		
	-- Kill dead timers dead
	local intAlive = 0
	for k, tblTimer in pairs(tblAchievementTimers) do
		if (tblTimer["TimeLeft"] <= 0) then
			tblAchievementTimers[k] = nil
		else
			intAlive = intAlive + 1
		end
	end
	intNumAchievementTimers = intAlive
	
	--Add in slight offsets for outlined text to try stop overlap
	local intTitleOutlineOffset = 0
	local intObjectiveOutlineOffset = 0
	if (db.Fonts.AchievementTitleFontOutline == "THICKOUTLINE") then
		intTitleOutlineOffset = 2
	elseif (db.Fonts.AchievementTitleFontOutline == "OUTLINE") then
		intTitleOutlineOffset = 1
	end
	if (db.Fonts.AchievementObjectiveFontOutline == "THICKOUTLINE") then
		intObjectiveOutlineOffset = 1.5
	elseif (db.Fonts.AchievementObjectiveFontOutline == "OUTLINE") then
		intObjectiveOutlineOffset = 0.5
	end

	
	local intLargestWidth = 0
	-- Show title if enabled
	if (db.AchievementMinion.ShowTitle == true and (db.AchievementMinion.HideTitleWhenNoTracking == false or (db.AchievementMinion.HideTitleWhenNoTracking == true and (GetNumTrackedAchievements() > 0)))) then
		fraAchievementMinionAnchor.objFontString:SetFont(LSM:Fetch("font", db.Fonts.AchievementTitleFont), db.Fonts.AchievementTitleFontSize, db.Fonts.AchievementTitleFontOutline)
		fraAchievementMinionAnchor.objFontString:SetText(strInfoColour .. L["Achivement Tracker Title"])
		intLargestWidth = fraAchievementMinionAnchor.objFontString:GetWidth()
	else
		fraAchievementMinionAnchor.objFontString:SetText("")
		intLargestWidth = 100
	end			
	fraAchievementMinionAnchor:SetWidth(db.AchievementMinion.MinionWidth)	
			

	
	
	-- Create main minions buttons and set text etc
	local intYPosition = db.Fonts.AchievementTitleFontSize
	local intOffset = 0

	for _, intID in ipairs({GetTrackedAchievements()}) do
		local strTitle, strObjectives = self:GetAchievementText(intID)

		-- Get a button for achivement
		objButton = self:GetAchievementMinionButton()
		objButton:SetPoint("TOPLEFT", fraAchievementMinionAnchor, "TOPLEFT", 0, -intYPosition);
		objButton.intID = intID
		
		-- Set buttons title text
		objButton.objFontString1:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, 0);
		objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.AchievementTitleFont), db.Fonts.AchievementTitleFontSize, db.Fonts.AchievementTitleFontOutline)
		objButton.objFontString1:SetText(strTitle)
		
		objButton:SetWidth(db.AchievementMinion.MinionWidth)

		-- Find out if string is larger then the current largest string
		if (objButton.objFontString1:GetWidth() > intLargestWidth) then
			intLargestWidth = objButton.objFontString1:GetWidth()
		end
		objButton.objFontString1:SetWidth(db.AchievementMinion.MinionWidth)

		intOffset = objButton.objFontString1:GetHeight() + intTitleOutlineOffset + intObjectiveOutlineOffset
			
		-- Set buttons objective text
		objButton.objFontString2:SetPoint("TOPLEFT", objButton, "TOPLEFT", 0, -intOffset);
		objButton.objFontString2:SetFont(LSM:Fetch("font", db.Fonts.AchievementObjectiveFont), db.Fonts.AchievementObjectiveFontSize, db.Fonts.AchievementObjectiveFontOutline)
		objButton.objFontString2:SetText(strObjectives)

		-- Find out if string is larger then the current largest string
		if (objButton.objFontString2:GetWidth() > intLargestWidth) then
			intLargestWidth = objButton.objFontString2:GetWidth()
		end
		objButton.objFontString2:SetWidth(db.AchievementMinion.MinionWidth)
		
		intOffset = intOffset + objButton.objFontString2:GetHeight() + intTitleOutlineOffset + intObjectiveOutlineOffset

		objButton:SetHeight(intOffset)
		intYPosition = intYPosition + intOffset
		tinsert(tblAchievementMinionButtons, objButton)
	end
	
	-- Auto collapse
	if (db.AchievementMinion.MinionCollapseToLeft == true) then
		if (intLargestWidth < db.AchievementMinion.MinionWidth) then
			fraAchievementMinionAnchor:SetWidth(intLargestWidth)
			
			for k, objButton in pairs(tblAchievementMinionButtons) do
				objButton.objFontString1:SetWidth(intLargestWidth)
				objButton:SetWidth(intLargestWidth)
			end
		end
	end

	blnAchievementMinionIsUpdating = false
end


-- Button functions
function SorhaQuestLog:GetQuestMinionButton()
	objButton = self:GetLogButton()
	objButton:SetParent(fraQuestMinionAnchor)
	-- Create scripts
	objButton:RegisterForClicks("AnyUp")
	objButton:SetScript("OnClick", function(self, button)
		blnWasAClick = true
		if (button == "LeftButton") then
			if (self.isHeader == 1) then
				if (self.isCollapsed == 1) then
					ExpandQuestHeader(self.LogPosition)
				else
					CollapseQuestHeader(self.LogPosition)
				end
			else
				if (IsShiftKeyDown()) then
					ChatEdit_InsertLink(GetQuestLink(self.LogPosition))
				else
					ShowUIPanel(QuestLogFrame);
					SelectQuestLogEntry(self.LogPosition);
					QuestLog_SetSelection(self.LogPosition);
					QuestLog_Update();
				end
			end
		else
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.LogPosition);
			if (self.isHeader == 1) then
				if (db.ZonesAndQuests.AllowHiddenQuests == true) then
					SorhaQuestLog:DisplayRightClickMenu(self)
				end
			else
				if (db.ZonesAndQuests.AllowHiddenQuests == true) then
					if (IsQuestWatched(self.LogPosition) == nil) then
						if (GetNumQuestWatches() >= 25) then
							UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, 25), 1.0, 0.1, 0.1, 1.0);
						else
							AddQuestWatch(self.LogPosition)
						end
					else
						RemoveQuestWatch(self.LogPosition)
					end
					SorhaQuestLog:UpdateQuestMinion()
				end
			end
		end
	end)
	objButton:SetScript("OnEnter", function(self)
		if (not(self.isHeader)) then
			SelectQuestLogEntry(self.LogPosition);
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(self.LogPosition);
			if (db.Minion.MoveItemsAndTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, -50);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, -50);
			end
			
			local strQuestDescription, strQuestObjectives = GetQuestLogQuestText();
			local objColour = GetQuestDifficultyColor(level);
			GameTooltip:SetText(questTitle, 0, 1, 0, 1);
			GameTooltip:AddLine(strQuestObjectives, 0.5, 0.5, 0.5, 1);
			
			local strQuestTag = ""
			if (isDaily) then
				strQuestTag = strQuestTag .. L["Daily"] .. " "
			end				
			if (questTag) then
				strQuestTag = strQuestTag .. questTag
				if (suggestedGroup > 0) then
					strQuestTag = strQuestTag .. " (" .. suggestedGroup .. ")"
				end
			end				
			GameTooltip:AddLine(strQuestTag, 1, 0, 0, 1);
			
			local intPartyMembers = GetNumPartyMembers();
			local blnIsOnQuest = false
			if (intPartyMembers > 0) then
				GameTooltip:AddLine(L["Party members on quest:"], 0, 1, 0, 1);
				for k = 1, intPartyMembers do
					blnIsOnQuest = IsUnitOnQuest(self.LogPosition, "party" .. k);
					if (blnIsOnQuest == 1) then
						GameTooltip:AddLine(UnitName("party" .. k), 1, 1, 1, 1);
					end
				end				
			end
			
			GameTooltip:Show();
		end
	end)
	objButton:SetScript("OnLeave", function(self) 
		GameTooltip:Hide() 
	end)
	
	return objButton
end

function SorhaQuestLog:GetAchievementMinionButton()
	objButton = self:GetLogButton()
	objButton:SetParent(fraAchievementMinionAnchor)
	
	-- Create scripts
	objButton:RegisterForClicks("AnyUp")
	objButton:SetScript("OnClick", function(self, button)
		blnWasAClick = true
		if (button == "LeftButton") then
			if not(AchievementFrame) then
				AchievementFrame_LoadUI();
			end

			if not(AchievementFrame:IsShown()) then
				AchievementFrame_ToggleAchievementFrame();
			end
			AchievementFrame_SelectAchievement(self.intID);
		else
			RemoveTrackedAchievement(self.intID)
		end
	end)
	objButton:SetScript("OnEnter", function(self)
		local strTitle, strObjectives = SorhaQuestLog:GetAchievementText(self.intID)
		if (db.AchievementMinion.MoveTooltipsRight == true) then
				GameTooltip:SetOwner(self, "ANCHOR_RIGHT", 0, -50);
			else 
				GameTooltip:SetOwner(self, "ANCHOR_LEFT", 0, -50);
		end
		GameTooltip:SetText(strTitle, 0, 1, 0, 1);
		GameTooltip:AddLine(strObjectives, 0.5, 0.5, 0.5, 1);
			
		GameTooltip:Show();
	end)
	objButton:SetScript("OnLeave", function(self) 
		GameTooltip:Hide() 
	end)
	
	return objButton
end

function SorhaQuestLog:GetLogButton()
	local objButton = tremove(tblLogButtonCache)
	
	if (objButton == nil) then
		intNumberOfLogButtons = intNumberOfLogButtons + 1
		local strButtonName = strButtonPrefix .. intNumberOfLogButtons
		
		-- Create button
		objButton = self:doCreateLooseFrame("BUTTON", strButtonName, UIParent, 10, 10, 1, "LOW", 1, 1)
		objButton:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,	insets = {left = 5, right = 3, top = 3, bottom = 5}})
		objButton:SetBackdropColor(0, 0, 0, 0)
		objButton:SetBackdropBorderColor(0, 0, 0, 0)		

		-- Create buttons variables
		objButton.isHeader = nil
		objButton.isCollapsed = nil
		objButton.LogPosition = 0
		objButton.intID = 0

		-- Create Primary fontstring
		objButton.objFontString1 = objButton:CreateFontString(nil, "OVERLAY");
		objButton.objFontString1:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 11, db.Fonts.HeaderFontOutline)
		objButton.objFontString1:SetJustifyH("LEFT")
		objButton.objFontString1:SetJustifyV("TOP")
		objButton.objFontString1:SetText("");
		objButton.objFontString1:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		objButton.objFontString1:SetShadowOffset(1, -1)
		
		-- Create Secondary fontstring
		objButton.objFontString2 = objButton:CreateFontString(nil, "OVERLAY");
		objButton.objFontString2:SetFont(LSM:Fetch("font", db.Fonts.HeaderFont), 11, db.Fonts.HeaderFontOutline)
		objButton.objFontString2:SetJustifyH("LEFT")
		objButton.objFontString2:SetJustifyV("TOP")
		objButton.objFontString2:SetText("");
		objButton.objFontString2:SetShadowColor(0.0, 0.0, 0.0, 1.0)
		objButton.objFontString2:SetShadowOffset(1, -1)
	end
	
	objButton:Show()
	objButton:EnableMouse(true);
	
	return objButton
end

function SorhaQuestLog:RecycleLogButton(objButton)
	objButton:Hide()
	objButton:EnableMouse(false);
	
	if (objButton.ItemButton ~= nil) then
		self:RecycleItemButton(objButton.ItemButton)
		objButton.ItemButton = nil
	end
	objButton.objFontString1:SetText("")
	objButton.objFontString1:SetWidth(0)
	objButton.objFontString1:SetHeight(0)
	objButton.objFontString1:ClearAllPoints()
	
	objButton.objFontString2:SetText("")
	objButton.objFontString2:SetWidth(0)
	objButton.objFontString2:SetHeight(0)	
	objButton.objFontString2:ClearAllPoints()
	
	objButton:SetScript("OnEnter", nil);
	objButton:SetScript("OnLeave", nil);
	objButton:SetScript("OnClick", nil);
	
	objButton:ClearAllPoints()
	objButton.isHeader = nil
	objButton.isCollapsed = nil
	objButton.LogPosition = 0
	objButton.intID = 0
	objButton.intOffset = 0

	tinsert(tblLogButtonCache, objButton)
end

function SorhaQuestLog:GetItemButton(objItem, intLogPosition)
	objButton = tremove(tblItemButtonCache)
	if (objButton == nil) then
		intNumberOfItemButtons = intNumberOfItemButtons + 1
		objButton = CreateFrame("BUTTON", strItemButtonPrefix .. intNumberOfItemButtons, UIParent, "WatchFrameItemButtonTemplate");		
	end
	
	objButton:Show()
	objButton:SetID(intLogPosition)
	objButton:SetScale(db.Minion.ItemButtonScale)
	
	SetItemButtonTexture(objButton, objItem["Item"])
	SetItemButtonTextureVertexColor(objButton, 1.0, 1.0, 1.0)
	SetItemButtonCount(objButton, objItem["Charges"]);
	WatchFrameItem_UpdateCooldown(objButton);

	if (ItemHasRange(objItem["Link"]) ~= nil) then
		objButton.rangeTimer = -1;
		objButton:SetScript("OnUpdate", function(self, elapsed)
			self.rangeTimer = self.rangeTimer - elapsed
			if self.rangeTimer <= 0 then
				if IsQuestLogSpecialItemInRange(self:GetID()) == 0 then
					SetItemButtonTextureVertexColor(self, 0.8, 0.1, 0.1)
				else
					SetItemButtonTextureVertexColor(self, 1.0, 1.0, 1.0)
				end
				self.rangeTimer = TOOLTIP_UPDATE_TIME
			end
		end)

	end
	return objButton
end

function SorhaQuestLog:GetRemoteQuestButton()
	objButton = tremove(tblRemoteButtonCache)
	if (objButton == nil) then
		intNumberOfRemoteButtons = intNumberOfRemoteButtons + 1
		objButton = CreateFrame("SCROLLFRAME", strRemoteButtonPrefix .. intNumberOfRemoteButtons, UIParent, "WatchFrameAutoQuestPopUpTemplate");		
	end
	
	objButton:Show()
	return objButton
end

function SorhaQuestLog:RecycleItemButton(objButton)
	objButton:SetParent(UIParent)
	objButton:ClearAllPoints()
	objButton:Hide()
	objButton:SetScript("OnUpdate", nil)	
	tinsert(tblItemButtonCache, objButton)
end

function SorhaQuestLog:RecycleRemoteQuestButton(objButton)
	objButton:SetParent(UIParent)
	objButton:SetScale(1)
	objButton:ClearAllPoints()

	objButton:SetScript("OnUpdate", nil)	
	objButton.index = 0
	objButton:SetHeight(1)
	objButton.totalTime = 0
	objButton.slideInTime = 0
	objButton.type=""
	objButton.ScrollChild.QuestionMark:Hide();
	objButton.ScrollChild.Exclamation:Hide();
	objButton.ScrollChild.TopText:SetText("");
	objButton.ScrollChild.BottomText:SetText("");
	objButton.ScrollChild.BottomText:Hide();
	objButton.ScrollChild.QuestName:SetText("");
	objButton.ScrollChild.Flash:Hide();
	objButton.questId = 0;

	objButton:Hide()

	tinsert(tblRemoteButtonCache, objButton)
end


-- Quest information functions
function SorhaQuestLog:GetQuestText(QuestInstance)
	local strLevelColor = ""
	local strTitleColor = ""
	
	-- Get quest level colour
	if (db.ZonesAndQuests.QuestLevelColouringSetting == "Level") then
		local objColour = GetQuestDifficultyColor(QuestInstance["Level"]);	
		strLevelColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	
	elseif (db.ZonesAndQuests.QuestLevelColouringSetting == "Completion") then
		strLevelColor = self:GetQuestCompletionColour(QuestInstance["CompletionLevel"])
	
	elseif (db.ZonesAndQuests.QuestLevelColouringSetting == "Done/Undone") then
		if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
			strLevelColor = strObjective100Colour
		else
			strLevelColor = strObjective00to24Colour
		end
	
	else
		strLevelColor = strQuestLevelColour
	end
	
	-- Get quest title colour
	if (db.ZonesAndQuests.QuestTitleColouringSetting == "Level") then
		local objColour = GetQuestDifficultyColor(QuestInstance["Level"]);	
		strTitleColor = format("|c%02X%02X%02X%02X", 255, objColour.r * 255, objColour.g * 255, objColour.b * 255);
	
	elseif (db.ZonesAndQuests.QuestTitleColouringSetting == "Completion") then
		strTitleColor = self:GetQuestCompletionColour(QuestInstance["CompletionLevel"])
	
	elseif (db.ZonesAndQuests.QuestTitleColouringSetting == "Done/Undone") then
		if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
			strTitleColor = strObjective100Colour
		else
			strTitleColor = strObjective00to24Colour
		end
	
	else
		strTitleColor = strQuestTitleColour
	end
	
	
	-- Quest level
	local strQuestReturnText = strLevelColor .. "[" .. QuestInstance["Level"]
	
	-- PVP, RAID, Group, etc tags
	if (QuestInstance["Tag"] ~= nil) then
		if (db.ZonesAndQuests.FullLengthQuestTags == true and dicLongQuestTags[QuestInstance["Tag"]] ~= nil) then
			strQuestReturnText = strQuestReturnText .. " " .. dicLongQuestTags[QuestInstance["Tag"]]
		elseif (db.ZonesAndQuests.FullLengthQuestTags == false and dicQuestTags[QuestInstance["Tag"]] ~= nil) then
			strQuestReturnText = strQuestReturnText .. dicQuestTags[QuestInstance["Tag"]]
		end
	end
	
	if (QuestInstance["SuggestedGroup"] > 0) then
		strQuestReturnText = strQuestReturnText .. QuestInstance["SuggestedGroup"]
	end

	if (QuestInstance["IsDaily"]) then
		if (db.ZonesAndQuests.FullLengthQuestTags == true) then
			strQuestReturnText = strQuestReturnText .. " " .. dicLongQuestTags["Daily"]
		else
			strQuestReturnText = strQuestReturnText .. dicQuestTags["Daily"]
		end
	end

	strQuestReturnText = strQuestReturnText .. "]|r " .. strTitleColor .. QuestInstance["Title"] .. "|r"
	
	-- Completion/failed etc tag
	if (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] < 0) then 
		strQuestReturnText = strQuestReturnText .. strQuestStatusFailed .. L[" (Failed)"] .. "|r"
	elseif (QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0) then
		strQuestReturnText = strQuestReturnText .. strQuestStatusDone .. L[" (Done)"] .. "|r"
	elseif (#QuestInstance["ObjectiveList"] == 0) then
		strQuestReturnText = strQuestReturnText .. strQuestStatusGoto .. L[" (goto)"] .. "|r"
	end
	
	-- Hidden tag
	if (QuestInstance["IsHidden"] == true and db.ZonesAndQuests.AllowHiddenQuests == true) then
		strQuestReturnText = strQuestReturnText .. strHeaderColour .. L[" (Hidden)"] .. "|r"
	end 
	
	local strObjectivesReturnText = ""
	
	-- Objectives
	if (#QuestInstance["ObjectiveList"] == 0) then
		-- If no objective and show descriptions on display quest description
		if (db.ZonesAndQuests.ShowDescWhenNoObjectives == true) then
			if (not(QuestInstance["IsComplete"] and QuestInstance["IsComplete"] > 0)) then
				strObjectivesReturnText = strObjectivesReturnText .. strObjectiveDescriptionColour .. " - " .. QuestInstance["ObjectiveDescription"] .. "|r\n"
			else
				if (db.ZonesAndQuests.HideCompletedObjectives == false) then
					strObjectivesReturnText = strObjectivesReturnText .. strObjectiveDescriptionColour .. " - " .. QuestInstance["ObjectiveDescription"] .. "|r\n"
				end
			end	
		end
	else
		-- For each objective in quest
		for k, ObjectiveInstance in pairs(QuestInstance["ObjectiveList"]) do
			if not(db.ZonesAndQuests.HideCompletedObjectives == true and ObjectiveInstance["Status"] == 1) then
				local strObjectiveGradualColour = "|cffffffff"
				local strObjectiveTitleColourOutput = strObjectiveTitleColour
				local strObjectiveStatusColourOutput = strObjectiveStatusColour
				
				-- If somethings uses gradual colours get colour
				if (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion" or db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion") then
					strObjectiveGradualColour = self:GetQuestCompletionColour(ObjectiveInstance["CompletionLevel"])
				end
				
				-- Decide on quest title colour
				if (db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Completion") then
					strObjectiveTitleColourOutput = strObjectiveGradualColour
				elseif (db.ZonesAndQuests.ObjectiveTitleColouringSetting == "Done/Undone") then
					if (ObjectiveInstance["Status"] == nil) then
						strObjectiveTitleColourOutput = strObjective00to24Colour
					else
						strObjectiveTitleColourOutput = strObjective100Colour
					end
				end	

				-- Decide on quest status (0/1 etc) colour
				if (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Completion") then
					strObjectiveStatusColourOutput = strObjectiveGradualColour
				elseif (db.ZonesAndQuests.ObjectiveStatusColouringSetting == "Done/Undone") then
					if (ObjectiveInstance["Status"] == nil) then
						strObjectiveStatusColourOutput = strObjective00to24Colour
					else
						strObjectiveStatusColourOutput = strObjective100Colour
					end
				end				
				
				-- Depending on quest type display it in a certain way
				if (ObjectiveInstance["Type"] == nil) then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"		
					
				elseif (ObjectiveInstance["Type"] == "event") then
					if (ObjectiveInstance["Status"] == nil) then
						strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"
					else
						strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r" .. strObjective100Colour .. " (Done)|r"
					end
					
				elseif (ObjectiveInstance["Type"] == "log") then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. "|r"
					
				elseif (ObjectiveInstance["Type"] == "reputation") then
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. ": |r" .. strObjectiveStatusColourOutput .. ObjectiveInstance["Got"] .. " / " .. ObjectiveInstance["Need"] .. "|r"	
					
				else
					strOutput = strObjectiveTitleColourOutput .. " - " .. ObjectiveInstance["Text"] .. ": |r" .. strObjectiveStatusColourOutput .. ObjectiveInstance["Got"] .. "/" .. ObjectiveInstance["Need"] .. "|r"
				end
				
				strObjectivesReturnText = strObjectivesReturnText .. strOutput .. "\n"
			end
		end
	end
	strQuestReturnText = strtrim(strQuestReturnText)
	strObjectivesReturnText = strtrim(strObjectivesReturnText)
	
	return {["Title"] = strQuestReturnText, ["Objectives"] = strObjectivesReturnText}
end

function SorhaQuestLog:GetQuestCompletionColour(dblPercent)
	if (dblPercent < 0.25) then
		return strObjective00to24Colour
	elseif (dblPercent >= 0.25 and dblPercent < 0.50) then
		return strObjective25to49Colour
	elseif (dblPercent >= 0.50 and dblPercent < 0.75) then
		return strObjective50to74Colour
	elseif (dblPercent >= 0.75 and dblPercent < 1) then
		return strObjective75to99Colour
	else
		return strObjective100Colour
	end
end

function SorhaQuestLog:CheckQuestStateChange()
	local blnQuestNotFound = false
	local blnObjectiveCompleted = false
	local blnQuestCompleted = false
	local dicOldQuests = {}
	local dicCurQuests = {}	
	
	-- Compile list of old quests
	for k, ZoneInstance in pairs(oldQuestInfo["Zones"]) do -- Get list of old quests
		for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
			tinsert(dicOldQuests, QuestInstance)
		end
	end	
	
	-- Compile list of new quests
	for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do -- Get list of current quests
		for k2, QuestInstance in pairs(ZoneInstance["Quests"]) do
			tinsert(dicCurQuests, QuestInstance)
		end
	end
	
	-- For each quest in the old compare it to each quest in the new.
		-- When they match:
			-- Check each old objective against each new objective to check for status changes
			-- Check old quests status against new quest status
	for oldQuestKey, oldQuestInstance in pairs(dicOldQuests) do
		for curQuestKey, curQuestInstance in pairs(dicCurQuests) do
		
			if (oldQuestInstance["ID"] == curQuestInstance["ID"]) then -- If is same quest
				if (blnObjectiveCompleted == false) then -- If havn't found changed objective look for one
				
					for oldObjectiveKey, oldObjectiveInstance in pairs(oldQuestInstance["ObjectiveList"]) do
						for curObjectiveKey, curObjectiveInstance in pairs(curQuestInstance["ObjectiveList"]) do
						
							if (oldObjectiveInstance["ID"] == curObjectiveInstance["ID"]) then
								if (oldObjectiveInstance["Status"] == nil and curObjectiveInstance["Status"] == 1) then
									blnObjectiveCompleted = true
									break
								end
							end
						end
						if (blnObjectiveCompleted == true) then
							break
						end
					end
				end
				if (oldQuestInstance["IsComplete"] == nil and curQuestInstance["IsComplete"] == 1) then -- Has quest been completed since last
					blnQuestCompleted = true
					break
				end
			end
		end	
		if (blnQuestCompleted == true) then
			break
		end
	end
	
	-- Quest has been completed
	if (blnQuestCompleted == true) then
		return "QuestDone"
	end
	
	-- Objective has been completed
	if (blnObjectiveCompleted == true) then
		return "ObjectiveDone"
	end
	
	return ""
end

function SorhaQuestLog:GetQuestLogInformation()
	local intNoEntries, intNoQuests = GetNumQuestLogEntries();
	local i = 1
	local intEntrys = 0
	local QuestLogInfo = {["NoEntrys"] = 0, ["NoQuests"] = intNoQuests, ["Zones"] = {}}
	local ZoneEntry = nil
	local QuestEntry = {}
	local ObjectiveEntry = {}
	local ItemEntry = {}
	
	local intNumHidden = 0
	local intNumQuestsInZone = 0
	
	-- For each entry in the quest log
	for i = 1, intNoEntries, 1 do
		local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
		if (questTitle == nil) then
			questTitle = L["Unknown"]
		end
		
		if (isHeader) then
			-- Create new zone entry
			intEntrys = intEntrys + 1
			if not(ZoneEntry == nil) then
				ZoneEntry["NumHidden"] = intNumHidden
				ZoneEntry["NumQuests"] = intNumQuestsInZone
				tinsert(QuestLogInfo["Zones"], ZoneEntry)
				intNumHidden = 0
				intNumQuestsInZone = 0
			end
			ZoneEntry = {["Title"] = questTitle, ["Status"] = isCollapsed, ["LogPosition"] = i, ["ID"] = questID, ["HasHidden"] = false, ["NumHidden"] = intNumHidden, ["NumQuests"] = intNumQuestsInZone, ["Quests"] = {}}
		else
			-- Create new zone
			intNumQuestsInZone = intNumQuestsInZone + 1
			intEntrys = intEntrys + 1
			
			QuestEntry = {["Title"] = questTitle, ["Level"] = level, ["Tag"] = questTag, ["IsHidden"] = false, ["SuggestedGroup"] = suggestedGroup, ["IsComplete"] = isComplete, ["IsDaily"] = isDaily, ["LogPosition"] = i, ["ID"] = questID, ["QuestItem"] = nil, ["CompletionLevel"] = 0, ["ObjectiveList"] = {}, ["ObjectiveDescription"] = ""}
			local isWatched = IsQuestWatched(i)
			
			-- Hidden quest settings
			if (isWatched == nil) then
				intNumHidden = intNumHidden + 1
				QuestEntry["IsHidden"] = true
				ZoneEntry["HasHidden"] = true
			end
						
			-- Quest item
			local strLink, strItem, intCharges = GetQuestLogSpecialItemInfo(i);
			if (strItem) then
				ItemEntry = {["Link"] = strLink, ["Item"] = strItem, ["Charges"] = intCharges}
				QuestEntry["QuestItem"] = ItemEntry
			end
			
			SelectQuestLogEntry(i);
			local strQuestDescription, strQuestObjectiveDescription = GetQuestLogQuestText();
			local intNoOfObjectives = GetNumQuestLeaderBoards(i)
			local dblQuestCompletionPercent = 0
			local intNumUsedObjectives = 0
			
			QuestEntry["ObjectiveDescription"] = strQuestObjectiveDescription
			
			-- Objectives
			for k = 1, intNoOfObjectives, 1 do
				local leaderboardTxt, itemType, isDone = GetQuestLogLeaderBoard(k,i);
				if not(leaderboardTxt == nil) then
					intNumUsedObjectives = intNumUsedObjectives + 1
					local strObjText = ""
					local strGot = ""
					local strGetTo = ""
					local intCompletion = 0
					local intGot = 0
					local intNeeded = 1
				
					-- Change things based on objective type
					if (itemType == nil) then
						strObjText = leaderboardTxt
						
					elseif (itemType == "event") then
						strObjText = leaderboardTxt
						
					elseif (itemType == "log") then
						strObjText = leaderboardTxt		
						
					elseif (itemType == "reputation") then
						local y, z, strObjDesc, strRepHave, strRepNeeded = string.find(leaderboardTxt, "(.*):%s*(.*)%s*/%s*(.*)");
						if (strObjDesc == nil) then strObjText = "" else strObjText = strObjDesc end
						if (strRepHave == nil) then strGot = "" else strGot = strtrim(strRepHave) end
						if (strRepNeeded == nil) then strGetTo = "" else strGetTo = strtrim(strRepNeeded) end
						
						for k, RepInstance in pairs(dicRepLevels) do
							if (strGot == RepInstance["MTitle"] or strGot == RepInstance["FTitle"]) then
								intGot = RepInstance["Value"]
							end
							if (strGetTo == RepInstance["MTitle"] or strGetTo == RepInstance["FTitle"]) then
								intNeeded = RepInstance["Value"]
							end
						end
						intCompletion = intGot / intNeeded
					
					else
						local y, z, strObjDesc, intNumGot, intNumNeeded = string.find(leaderboardTxt, "(.*):%s*([%d]+)%s*/%s*([%d]+)");
						if (strObjDesc == nil) then strObjText = "" else strObjText = strObjDesc end
						if (intNumGot == nil) then strGot = 2 else strGot = intNumGot end
						if (intNumNeeded == nil) then strGetTo = 1 else strGetTo = intNumNeeded end
						intCompletion = strGot / strGetTo
					end
		
					if (isDone) then
						intCompletion = 1
					end
					dblQuestCompletionPercent = dblQuestCompletionPercent + intCompletion
					ObjectiveEntry = {["Text"] = strObjText, ["Got"] = strGot, ["Need"] = strGetTo, ["CompletionLevel"] = intCompletion, ["Type"] = itemType, ["Status"] = isDone, ["ID"] = k}
					tinsert(QuestEntry["ObjectiveList"], ObjectiveEntry)
				end
			end
			if (intNoOfObjectives < 1) then
				dblQuestCompletionPercent = 1
			else
				dblQuestCompletionPercent = dblQuestCompletionPercent / intNumUsedObjectives
			end
			QuestEntry["CompletionLevel"] = dblQuestCompletionPercent
			tinsert(ZoneEntry["Quests"], QuestEntry)
		end
	end
	if (ZoneEntry ~= nil) then
		ZoneEntry["NumHidden"] = intNumHidden
		ZoneEntry["NumQuests"] = intNumQuestsInZone
		tinsert(QuestLogInfo["Zones"], ZoneEntry)
	end
	QuestLogInfo["NoEntrys"] = intEntrys
	
	return QuestLogInfo
end


-- Remote Quest functions
function RemoteQuestButton_OnUpdate(frame, timestep)
	local height = 72;
	local scrollStart = 65;
	local scrollEnd = -9;
	local offset = 0
	
	-- The first pop-up needs to include the WATCHFRAME_TYPE_OFFSET in the animation
	if (frame.index == 1) then
		height = height + offset;
		scrollEnd = scrollEnd - offset;
	end
	
	frame.totalTime = frame.totalTime+timestep;
	if (frame.totalTime > frame.slideInTime) then
		frame.totalTime = frame.slideInTime;
	end
	
	local scrollPos = scrollEnd;
	if (frame.slideInTime and frame.slideInTime > 0) then
		height = height*(frame.totalTime/frame.slideInTime);
		scrollPos = scrollStart + (scrollEnd-scrollStart)*(frame.totalTime/frame.slideInTime);
	end
	frame:SetHeight(height);
	frame:SetVerticalScroll(floor(scrollPos+0.5));
	if (frame.totalTime >= frame.slideInTime) then
		frame:SetScript("OnUpdate", nil);
		frame.ScrollChild.Shine:Show();
		frame.ScrollChild.IconShine:Show();
		frame.ScrollChild.Shine.Flash:Play();
		frame.ScrollChild.IconShine.Flash:Play();
	end
end


-- Achievement information functions
function SorhaQuestLog:GetAchievementText(AchievementID)
	local strOutputTitle = strAchievementTitleColour
	local strOutputObjectives = strAchievementObjectiveColour
	
	local id, name, points, completed, month, day, year, description, flags, icon, rewardText = GetAchievementInfo(AchievementID)
	strOutputTitle =  strOutputTitle .. name .. "|r"
	
	
	if (tblAchievementTimers[AchievementID]) then
		tblAchievementTimers[AchievementID]["TimeLeft"] = tblAchievementTimers[AchievementID]["Duration"] - (GetTime() - tblAchievementTimers[AchievementID]["StartTime"])
		
		local r,g,b = GetTimerTextColor(tblAchievementTimers[AchievementID]["Duration"], (GetTime() - tblAchievementTimers[AchievementID]["StartTime"]))
		local strTimeColour = format("|c%02X%02X%02X%02X", 255, r * 255, g * 255, b * 255);
		strOutputTitle = strOutputTitle .. strTimeColour .. " - " .. SecondsToTime(tblAchievementTimers[AchievementID]["TimeLeft"]) .. "|r"
	end
		
	local intNumParts = GetAchievementNumCriteria(AchievementID)
	if (intNumParts > 0) then
		for i = 1, intNumParts, 1 do
			local description, type, completed, quantity, requiredQuantity, characterName, flags, assetID, quantityString, criteriaID = GetAchievementCriteriaInfo(AchievementID, i)
			if (completed == false) then
				if string.match(quantityString, ".+/.+") then
					strOutputObjectives = strOutputObjectives .. "- " .. quantityString .. "\n"			
				else
					strOutputObjectives = strOutputObjectives .. "- " .. description .. "\n"
				end
			end
		end
	else
		strOutputObjectives = strOutputObjectives .. "- " .. description
	end
	strOutputObjectives = string.trim(strOutputObjectives)
	strOutputObjectives = strOutputObjectives .. "|r"
	
	return strOutputTitle, strOutputObjectives
end

function GetTimerTextColor(duration, elapsed) -- Thankyou blizzard >.>
	local START_PERCENTAGE_YELLOW = .66
	local START_PERCENTAGE_RED = .33
	
	local percentageLeft = 1 - ( elapsed / duration )
	if ( percentageLeft > START_PERCENTAGE_YELLOW ) then
		return 1, 1, 1	
	elseif ( percentageLeft > START_PERCENTAGE_RED ) then -- Start fading to yellow by eliminating blue
		local blueOffset = (percentageLeft - START_PERCENTAGE_RED) / (START_PERCENTAGE_YELLOW - START_PERCENTAGE_RED);
		return 1, 1, blueOffset;
	else
		local greenOffset = percentageLeft / START_PERCENTAGE_RED; -- Fade to red by eliminating green
		return 1, greenOffset, 0;
	end
end


-- Colour functions
function SorhaQuestLog:doColourChange() --Changes the addons colours on the fly as changed
	self:SetupColours()
	if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false) then
		if (blnIsUpdating == false) then
			blnIsUpdating = true
			self:ScheduleTimer("UpdateQuestMinion", 0.1)
		end
	end
	if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
		if (blnAchievementMinionIsUpdating == false) then
			self:UpdateAchievementMinion()
		end
	end
	
end

function SorhaQuestLog:SetupColours() -- Sets the colour strings from profile settings
	strInfoColour = format("|c%02X%02X%02X%02X", 255, db.Colours.InfoColour.r * 255, db.Colours.InfoColour.g * 255, db.Colours.InfoColour.b * 255);
	strHeaderColour = format("|c%02X%02X%02X%02X", 255, db.Colours.HeaderColour.r * 255, db.Colours.HeaderColour.g * 255, db.Colours.HeaderColour.b * 255);	
	strQuestStatusFailed = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestStatusFailedColour.r * 255, db.Colours.QuestStatusFailedColour.g * 255, db.Colours.QuestStatusFailedColour.b * 255);
	strQuestStatusDone = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestStatusDoneColour.r * 255, db.Colours.QuestStatusDoneColour.g * 255, db.Colours.QuestStatusDoneColour.b * 255);
	strQuestStatusGoto = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestStatusGotoColour.r * 255, db.Colours.QuestStatusGotoColour.g * 255, db.Colours.QuestStatusGotoColour.b * 255);
	strQuestLevelColour = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestLevelColour.r * 255, db.Colours.QuestLevelColour.g * 255, db.Colours.QuestLevelColour.b * 255);
	strQuestTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.QuestTitleColour.r * 255, db.Colours.QuestTitleColour.g * 255, db.Colours.QuestTitleColour.b * 255);
	strObjectiveTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveTitleColour.r * 255, db.Colours.ObjectiveTitleColour.g * 255, db.Colours.ObjectiveTitleColour.b * 255);
	strObjectiveStatusColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveStatusColour.r * 255, db.Colours.ObjectiveStatusColour.g * 255, db.Colours.ObjectiveStatusColour.b * 255);
	strObjective00to24Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective00PlusColour.r * 255, db.Colours.Objective00PlusColour.g * 255, db.Colours.Objective00PlusColour.b * 255);
	strObjective25to49Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective25PlusColour.r * 255, db.Colours.Objective25PlusColour.g * 255, db.Colours.Objective25PlusColour.b * 255);
	strObjective50to74Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective50PlusColour.r * 255, db.Colours.Objective50PlusColour.g * 255, db.Colours.Objective50PlusColour.b * 255);
	strObjective75to99Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.Objective75PlusColour.r * 255, db.Colours.Objective75PlusColour.g * 255, db.Colours.Objective75PlusColour.b * 255);
	strObjective100Colour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveDoneColour.r * 255, db.Colours.ObjectiveDoneColour.g * 255, db.Colours.ObjectiveDoneColour.b * 255);
	strObjectiveDescriptionColour = format("|c%02X%02X%02X%02X", 255, db.Colours.ObjectiveDescColour.r * 255, db.Colours.ObjectiveDescColour.g * 255, db.Colours.ObjectiveDescColour.b * 255);
	strAchievementTitleColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AchievementTitleColour.r * 255, db.Colours.AchievementTitleColour.g * 255, db.Colours.AchievementTitleColour.b * 255);
	strAchievementObjectiveColour = format("|c%02X%02X%02X%02X", 255, db.Colours.AchievementObjectiveColour.r * 255, db.Colours.AchievementObjectiveColour.g * 255, db.Colours.AchievementObjectiveColour.b * 255);
end


-- Anchor Functions
function SorhaQuestLog:doQuestMinionAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of quest minion anchor frame
	-- Quest Minion
	if (blnQuestMinionInitialized == false) then
		if (db.Main.HideSorhaQuestLog == false and db.Main.HideAll == false) then
			self:CreateQuestMinionLayout()
		end
	end	
	
	if (blnQuestMinionInitialized == true) then
		-- Enable/Disable movement
		if (db.Minion.MinionWindowLocked == false) then
			fraQuestMinionAnchor:EnableMouse(true)
		else
			fraQuestMinionAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Quest Minion
		if (db.Main.HideSorhaQuestLog == true or db.Main.HideAll == true) then
			fraQuestMinionAnchor:Hide()
		else
			fraQuestMinionAnchor:Show()
			if (blnIsUpdating == false) then
				blnIsUpdating = true
				self:ScheduleTimer("UpdateQuestMinion", 0.25)
			end
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraQuestMinionAnchor:ClearAllPoints()
			fraQuestMinionAnchor:SetPoint(db.Minion.MinionWindowLocation.Point, UIParent, db.Minion.MinionWindowLocation.RelativePoint, db.Minion.MinionWindowLocation.X, db.Minion.MinionWindowLocation.Y);
		end	
	end
end

function SorhaQuestLog:doQuestTimersAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of quest timers anchor frame
	-- Quest Timers
	if (blnTimerInitialized == false) then
		if (db.Main.ShowQuestTimerFrame == true and db.Main.HideAll == false) then
			self:CreateQuestTimerLayout()
		end
	end	
	
	if (blnTimerInitialized == true) then
		-- Enable/Disable movement	
		if (db.Timer.TimerWindowLocked == false) then
			fraQuestTimerAnchor:EnableMouse(true)
		else
			fraQuestTimerAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Quest Minion
		if (db.Main.ShowQuestTimerFrame == true and db.Main.HideAll == false) then
			fraQuestTimerAnchor:Show()
		else
			fraQuestTimerAnchor:Hide()
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraQuestTimerAnchor:ClearAllPoints()
			fraQuestTimerAnchor:SetPoint(db.Timer.TimerWindowLocation.Point, UIParent, db.Timer.TimerWindowLocation.RelativePoint, db.Timer.TimerWindowLocation.X, db.Timer.TimerWindowLocation.Y);
		end
	end
end

function SorhaQuestLog:doRemoteQuestsMinionAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of remote quests minion anchor frame
	-- Remote Quests
	if (blnRemoteQuestsMinionInitialized == false) then
		if (db.Main.ShowRemoteQuestsMinionFrame == true and db.Main.HideAll == false) then
			self:CreateRemoteQuestsLayout()
		end
	end	
	
	if (blnRemoteQuestsMinionInitialized == true) then
		-- Enable/Disable movement	
		if (db.RemoteQuests.MinionLocked == false) then
			fraRemoteQuestsAnchor:EnableMouse(true)
		else
			fraRemoteQuestsAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Quest Minion
		if (db.Main.ShowRemoteQuestsMinionFrame == true and db.Main.HideAll == false) then
			fraRemoteQuestsAnchor:Show()
			if (blnRemoteQuestsMinionIsUpdating == false) then
				self:UpdateRemoteQuestsMinion()
			end
		else
			fraRemoteQuestsAnchor:Hide()
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraRemoteQuestsAnchor:ClearAllPoints()
			fraRemoteQuestsAnchor:SetPoint(db.RemoteQuests.MinionLocation.Point, UIParent, db.RemoteQuests.MinionLocation.RelativePoint, db.RemoteQuests.MinionLocation.X, db.RemoteQuests.MinionLocation.Y);
		end
	end
end

function SorhaQuestLog:doAchievementMinionAnchorUpdate(blnMoveAnchors) -- Updates backgrounds and visibility of quest minion anchor frame
	-- Achievement Minion
	if (blnAchievementMinionInitialized == false) then
		if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
			self:CreateAchievementMinionLayout()
		end
	end
	
	if (blnAchievementMinionInitialized == true) then
		-- Enable/Disable movement	
		if (db.AchievementMinion.MinionLocked == false) then
			fraAchievementMinionAnchor:EnableMouse(true)
		else
			fraAchievementMinionAnchor:EnableMouse(false)
		end
		
		-- Show/Hide Minion
		if (db.Main.ShowAchievementMinionFrame == true and db.Main.HideAll == false) then
			fraAchievementMinionAnchor:Show()
			if (blnAchievementMinionIsUpdating == false) then
				self:UpdateAchievementMinion()
			end
		else
			fraAchievementMinionAnchor:Hide()
		end
		
		-- Set position to stored position
		if (blnMoveAnchors == true) then
			fraAchievementMinionAnchor:ClearAllPoints()
			fraAchievementMinionAnchor:SetPoint(db.AchievementMinion.MinionLocation.Point, UIParent, db.AchievementMinion.MinionLocation.RelativePoint, db.AchievementMinion.MinionLocation.X, db.AchievementMinion.MinionLocation.Y);
		end
	end
end

function SorhaQuestLog:doAnchorUpdates(blnMoveAnchors) -- Updates backgrounds and visibility of all anchor frames
	self:doQuestMinionAnchorUpdate(blnMoveAnchors)
	self:doQuestTimersAnchorUpdate(blnMoveAnchors)
	self:doRemoteQuestsMinionAnchorUpdate(blnMoveAnchors)
	self:doAchievementMinionAnchorUpdate(blnMoveAnchors)
end


-- Misc functions
function SorhaQuestLog:doHiddenQuestsUpdate()
	-- Show/Hide hidden quests button and move quest count text accordingly
	if (blnQuestMinionInitialized == true) then
		if (db.ZonesAndQuests.AllowHiddenQuests == true) then
			fraQuestMinionAnchor.buttonShowHidden:Show()	
			fraQuestMinionAnchor.buttonShowHiddenHover:Show()
			fraQuestMinionAnchor.objFontString:ClearAllPoints()
			fraQuestMinionAnchor.objFontString:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 16, -5);
			if (db.ZonesAndQuests.ShowAllQuests == true) then
				fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0, 0.3, 1, 1)
			else
				fraQuestMinionAnchor.buttonShowHidden:SetBackdropColor(0.5, 0.5, 0.5, 1)
			end
		else
			fraQuestMinionAnchor.buttonShowHidden:Hide()	
			fraQuestMinionAnchor.buttonShowHiddenHover:Hide()
			fraQuestMinionAnchor.objFontString:ClearAllPoints()
			fraQuestMinionAnchor.objFontString:SetPoint("TOPLEFT", fraQuestMinionAnchor, "TOPLEFT", 0, -5);
		end		
	end
end

function SorhaQuestLog:DisplayRightClickMenu(objButton)
	local objMenu = CreateFrame("Frame", "SorhaQuestLogMenuThing")
	local intLevel = 1
	local info = {}
	
	objMenu.displayMode = "MENU"
	objMenu.initialize = function(self, intLevel)
		if not intLevel then return end
		wipe(info)
		if intLevel == 1 then
			-- Create the title of the menu
			info.isTitle = 1
			info.text = L["Show/Hide Quests"]
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, intLevel)
			
			local intCurrentButton = 0
			local curZone = nil
			
			-- Get zone button belongs to
			for k, ZoneInstance in pairs(curQuestInfo["Zones"]) do
				if (ZoneInstance["LogPosition"] == objButton.LogPosition) then
					curZone = ZoneInstance
				end
			end

			-- Show/Hide buttons for each quest
			for k2, QuestInstance in pairs(curZone["Quests"]) do
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = nil
				info.text = QuestInstance["Title"]
				info.func = function()
					if (IsQuestWatched(QuestInstance["LogPosition"]) == nil) then
						if (GetNumQuestWatches() >= 25) then
							UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, 25), 1.0, 0.1, 0.1, 1.0);
						else
							AddQuestWatch(QuestInstance["LogPosition"])
						end
					else
						RemoveQuestWatch(QuestInstance["LogPosition"])
					end
					SorhaQuestLog:UpdateQuestMinion()
				end
				info.checked = IsQuestWatched(QuestInstance["LogPosition"])
				UIDropDownMenu_AddButton(info, intLevel)
			end
	
			-- Hide all button if not all hidden
			if not(curZone["NumHidden"] == curZone["NumQuests"])then
				info.text = L["Hide All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k2, QuestInstance in pairs(curZone["Quests"]) do
						RemoveQuestWatch(QuestInstance["LogPosition"])
					end
					SorhaQuestLog:UpdateQuestMinion()
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end
			
			-- Show all button if not all hidden
			if (curZone["NumHidden"] > 0)then
				info.text = L["Show All"]
				info.disabled = nil
				info.isTitle = nil
				info.notCheckable = 1
				info.func = function()
					for k2, QuestInstance in pairs(curZone["Quests"]) do
						if (GetNumQuestWatches() >= 25) then
							UIErrorsFrame:AddMessage(format(QUEST_WATCH_TOO_MANY, 25), 1.0, 0.1, 0.1, 1.0);
							break
						else
							AddQuestWatch(QuestInstance["LogPosition"])
						end
					end
					SorhaQuestLog:UpdateQuestMinion()
				end
				UIDropDownMenu_AddButton(info, intLevel)
			end

			-- Close menu item
			info.text = CLOSE
			info.func = function() CloseDropDownMenus() end
			info.checked = nil
			info.notCheckable = 1
			UIDropDownMenu_AddButton(info, intLevel)
		end
	end

	ToggleDropDownMenu(1, nil, objMenu, objButton, 0, 0)
end


-- Auto collapsing/expanding zones
function SorhaQuestLog:doHandleZoneChange()
	local blnNewZone = not(strZone == GetRealZoneText())
	strZone = GetRealZoneText()
	strSubZone = GetSubZoneText()
	blnIgnoreUpdateEvents = true
	local blnChanged = false
	
	if (db.ZonesAndQuests.CollapseOnLeave == true or db.ZonesAndQuests.ExpandOnEnter == true) then
		local numEntries, numQuests = GetNumQuestLogEntries();	
		for i = numEntries, 1, -1 do
			local questTitle, level, questTag, suggestedGroup, isHeader, isCollapsed, isComplete, isDaily, questID = GetQuestLogTitle(i);
			if (isHeader) then
				if (questTitle == strZone or questTitle == strSubZone) then
					if (db.ZonesAndQuests.ExpandOnEnter == true and isCollapsed) then
						ExpandQuestHeader(i)
						blnChanged = true
					end
				else		
					if (db.ZonesAndQuests.CollapseOnLeave == true and not(isCollapsed) and blnNewZone) then
						CollapseQuestHeader(i)
						blnChanged = true
					end				
				end
			end
		end
	end

	blnIgnoreUpdateEvents = false
	if (blnQuestMinionInitialized == true and blnChanged == true) then
		self:UpdateQuestMinion()
	end
end


-- Utility functions
function SorhaQuestLog:RemoveObject(objTable, objFind)
	for k, v in pairs(objTable) do
		if (v == objFind) then
			tremove(objTable,k)
			break
		end
	end
end

function SorhaQuestLog:doCreateLooseFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,intAlpha,finherit) -- Returns a frame thats not anchored
	local f = CreateFrame(strType,strName,fraParent,finherit)
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)	
	f:SetAlpha(intAlpha)
	return f  
end 

function SorhaQuestLog:doCreateFrame(strType,strName,fraParent,intWidth,intHeight,intScale,strStrata,intLevel,strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY,intAlpha,finherit) -- Returns an anchored frame
	local f = CreateFrame(strType,strName,fraParent,finherit)
	
	f:SetWidth(intWidth)
	f:SetHeight(intHeight)
	f:SetFrameStrata(strStrata)
	f:SetFrameLevel(intLevel)
	f:SetPoint(strPoint,fraRelativeFrame,strRelativePoint,intOffsetX,intOffsetY)
	f:SetAlpha(intAlpha)
	return f  
end 


-- Visibility functions
function SorhaQuestLog:ShowSorhaQuestLog()
	db.Main.HideAll = false
	self:doAnchorUpdates(false)
end

function SorhaQuestLog:HideSorhaQuestLog()
	db.Main.HideAll = true
	self:doAnchorUpdates(false)
end

function SorhaQuestLog:ToggleSorhaQuestLog()
	if (db.Main.HideAll == true) then
		self:ShowSorhaQuestLog()
	else
		self:HideSorhaQuestLog()
	end
end